@@ -29,11 +29,11 @@
 #include "Fit.h"
 #include "FitModelHandler.h"
 #include "fit_gsl.h"
-#include "../table/Table.h"
-#include "../matrix/Matrix.h"
-#include "../plot2D/QwtErrorPlotCurve.h"
-#include "../plot2D/FunctionCurve.h"
-#include "../plot2D/MultiLayer.h"
+#include <Table.h>
+#include <Matrix.h>
+#include <QwtErrorPlotCurve.h>
+#include <FunctionCurve.h>
+#include <MultiLayer.h>
 #include <ColorBox.h>
 
 #include <gsl/gsl_statistics.h>
@@ -105,8 +105,10 @@ gsl_multifit_fdfsolver * Fit::fitGSL(gsl_multifit_function_fdf f, int &iteration
 	else
 		T = gsl_multifit_fdfsolver_lmsder;
 
+	gsl_set_error_handler_off();
+
 	gsl_multifit_fdfsolver *s = gsl_multifit_fdfsolver_alloc (T, d_n, d_p);
-	gsl_multifit_fdfsolver_set (s, &f, d_param_init);
+	status = gsl_multifit_fdfsolver_set (s, &f, d_param_init);
 
 	size_t iter = 0;
 	bool inRange = true;
@@ -119,6 +121,12 @@ gsl_multifit_fdfsolver * Fit::fitGSL(gsl_multifit_function_fdf f, int &iteration
 		}
 	}
 
+	if (status){
+	    gsl_multifit_covar (s->J, 0.0, covar);
+	    iterations = 0;
+	    return s;
+	}
+
 	do{
 		iter++;
 		status = gsl_multifit_fdfsolver_iterate (s);
@@ -158,8 +166,11 @@ gsl_multimin_fminimizer * Fit::fitSimplex(gsl_multimin_function f, int &iteratio
 	//set all step sizes to 1 can be increased to converge faster
 	gsl_vector_set_all (ss, 10.0);
 
+	gsl_set_error_handler_off();
+
 	gsl_multimin_fminimizer *s_min = gsl_multimin_fminimizer_alloc (T, f.n);
 	status = gsl_multimin_fminimizer_set (s_min, &f, d_param_init, ss);
+
 	double size;
 	size_t iter = 0;
 	bool inRange = true;
@@ -172,6 +183,12 @@ gsl_multimin_fminimizer * Fit::fitSimplex(gsl_multimin_function f, int &iteratio
 		}
 	}
 
+	if (status) {
+	    iterations = 0;
+	    gsl_vector_free(ss);
+	    return s_min;
+	}
+
 	do{
 		iter++;
 		status = gsl_multimin_fminimizer_iterate (s_min);
@@ -205,8 +222,16 @@ gsl_multimin_fminimizer * Fit::fitSimplex(gsl_multimin_function f, int &iteratio
 bool Fit::setDataFromTable(Table *t, const QString& xColName, const QString& yColName, int from, int to)
 {
     if (Filter::setDataFromTable(t, xColName, yColName, from, to)){
-    	d_w = new double[d_n];
-    	for (int i=0; i<d_n; i++)//initialize the weighting data to 1.0
+    	if (d_w)
+			free(d_w);
+
+    	d_w = (double *)malloc(d_n*sizeof(double));
+        if (!d_w){
+            memoryErrorMessage();
+            return false;
+        }
+
+    	for (int i = 0; i < d_n; i++)//initialize the weighting data to 1.0
        		d_w[i] = 1.0;
 		return true;
 	} else
@@ -672,8 +697,8 @@ void Fit::showConfidenceLimits(double confidenceLevel)
 			return;
 		}
 		points = d_points;
-		double X0 = d_x[0];
-		double step = (d_x[d_n - 1] - X0)/(points - 1);
+		double X0 = d_from;
+		double step = fabs(d_from - d_to)/(points - 1);
 		for (int i = 0; i < points; i++)
 			X[i] = X0 + i*step;
 	} else
@@ -791,8 +816,8 @@ void Fit::showPredictionLimits(double confidenceLevel)
 			return;
 		}
 		points = d_points;
-		double X0 = d_x[0];
-		double step = (d_x[d_n - 1] - X0)/(points - 1);
+		double X0 = d_from;
+		double step = fabs(d_to - d_from)/(points - 1);
 		for (int i = 0; i < points; i++)
 			X[i] = X0 + i*step;
 	} else
@@ -824,12 +849,11 @@ void Fit::showPredictionLimits(double confidenceLevel)
 
 	double t = gsl_cdf_tdist_Pinv(1 - 0.5*(1 - confidenceLevel), d_n - d_p);
 	double x_mean = gsl_stats_mean(d_x, 1, d_n);
-	double sxx = 0.0;//gsl_stats_tss (d_x, 1, d_n);
+	double sxx = 0.0;
 	for (int i = 0; i < d_n; i++){
 		double dx = d_x[i] - x_mean;
 		sxx += dx*dx;
 	}
-
 	double mse = d_rss/double(d_n - d_p);
 	for (int i = 0; i < points; i++){
 		double x = X[i];
@@ -898,6 +922,8 @@ void Fit::fit()
 
 	QApplication::setOverrideCursor(Qt::WaitCursor);
 
+	removeDataSingularities();
+
 	struct FitData d_data = {d_n, d_p, d_x, d_y, d_w, this};
 
 	int status, iterations = d_max_iterations;
@@ -908,14 +934,16 @@ void Fit::fit()
 		f.params = &d_data;
 		gsl_multimin_fminimizer *s_min = fitSimplex(f, iterations, status);
 
-		// allocate memory and calculate covariance matrix based on residuals
-		gsl_matrix *J = gsl_matrix_alloc(d_n, d_p);
-		d_df(s_min->x,(void*)f.params, J);
-		gsl_multifit_covar (J, 0.0, covar);
-		chi_2 = s_min->fval;
+		if (!status) {
+		     // allocate memory and calculate covariance matrix based on residuals
+		     gsl_matrix *J = gsl_matrix_alloc(d_n, d_p);
+		     d_df(s_min->x,(void*)f.params, J);
+		     gsl_multifit_covar (J, 0.0, covar);
+		     chi_2 = s_min->fval;
 
-		// free previousely allocated memory
-		gsl_matrix_free (J);
+		     // free previousely allocated memory
+		     gsl_matrix_free (J);
+		}
 		gsl_multimin_fminimizer_free (s_min);
 	} else {
 		gsl_multifit_function_fdf f;
