@@ -698,7 +698,7 @@ void FitDialog::saveUserFunction()
 
 		QString filter = tr("QtiPlot fit model")+" (*.fit);;";
 		filter += tr("All files")+" (*.*)";
-		QString fn = QFileDialog::getSaveFileName(app, tr("QtiPlot") + " - " + tr("Save Fit Model As"),
+		QString fn = ApplicationWindow::getFileName(app, tr("QtiPlot") + " - " + tr("Save Fit Model As"),
                                 app->fitModelsPath + "/" + name, filter);
 		if (!fn.isEmpty()){
             QFileInfo fi(fn);
@@ -1338,7 +1338,7 @@ void FitDialog::enableWeightingParameters(int index)
 
 void FitDialog::closeEvent (QCloseEvent * e)
 {
-    if (d_preview_curve){
+    if (d_graph && d_preview_curve){
         d_preview_curve->detach();
         delete d_preview_curve;
         d_graph->replot();
@@ -1497,7 +1497,7 @@ void FitDialog::saveInitialGuesses()
     else {
 		QString filter = tr("QtiPlot fit model") + " (*.fit);;";
 		filter += tr("All files") + " (*.*)";
-		QString fn = QFileDialog::getSaveFileName(app, tr("QtiPlot") + " - " + tr("Save Fit Model As"),
+		QString fn = ApplicationWindow::getFileName(app, tr("QtiPlot") + " - " + tr("Save Fit Model As"),
                                 app->fitModelsPath + "/" + d_current_fit->objectName(), filter);
 		if (!fn.isEmpty()){
             QFileInfo fi(fn);
