@@ -39,6 +39,7 @@
 #include <QSplitter>
 #include <QDesktopServices>
 #include <QBuffer>
+#include <QLineEdit>
 
 #include <Graph.h>
 #include <Table.h>
@@ -57,7 +58,6 @@ class QTranslator;
 class QDockWidget;
 class QAction;
 class QActionGroup;
-class QLineEdit;
 class QTranslator;
 class QToolButton;
 class QShortcut;
@@ -129,7 +129,8 @@ public:
 	enum WindowType{NoWindow, TableWindow, MatrixWindow, MultiLayerWindow, NoteWindow, Plot3DWindow};
 	enum MatrixToTableConversion{Direct, XYZ, YXZ};
 	enum EndLineChar{LF, CRLF, CR};
-	enum Analysis{NoAnalysis, Integrate, Diff, FitLinear, FitGauss, FitLorentz, FitSigmoidal};
+	enum Analysis{NoAnalysis, Integrate, Diff, FitLinear, FitGauss, FitLorentz, FitSigmoidal, FitSlope};
+	enum LaTeXCompiler{MathTran, Local};
 
 	FolderListView *lv, *folders;
 	QDockWidget *logWindow;
@@ -160,6 +161,7 @@ public:
 	QList<QToolBar *> toolBarsList();
 
 	MdiSubWindow *activeWindow(WindowType type = NoWindow);
+	void setActiveWindow(MdiSubWindow *w){d_active_window = w;};
 	QMdiArea* workspace(){return d_workspace;};
 
 	int matrixUndoStackSize(){return d_matrix_undo_stack_size;};
@@ -181,6 +183,19 @@ public:
 	QTextEdit *scriptingConsole(){return console;};
 #endif
 
+	static QString imageFilter();
+	static QString getFileName(QWidget *parent = 0, const QString & caption = QString(),
+		const QString & dir = QString(), const QString & filter = QString(),
+		QString * selectedFilter = 0, bool save = true);
+
+	static void memoryAllocationError();
+	QColor readColorFromProject(const QString& name);
+
+	void enableMdiArea(bool on = true);
+	bool isMdiAreaEnabled(){return d_mdi_windows_area;};
+
+	void displayInfo(const QString& text){info->setText(text);};
+
 public slots:
 	//! \name Projects and Project Files
 	//@{
@@ -247,6 +262,8 @@ public slots:
 	MultiLayer* multilayerPlot(const QString& caption, int layers = 1, int rows = 1, int cols = 1);
 	//! used by the plot wizard
 	MultiLayer* multilayerPlot(const QStringList& colList);
+	MultiLayer* waterfallPlot();
+	MultiLayer* waterfallPlot(Table *t, const QStringList& list);
 	void connectMultilayerPlot(MultiLayer *g);
 	void addLayer();
 	void addInsetLayer(bool curves = false);
@@ -292,7 +309,7 @@ public slots:
 	void newSurfacePlot();
 	void editSurfacePlot();
 	void remove3DMatrixPlots(Matrix *m);
-	void updateMatrixPlots(MdiSubWindow *);
+	void updateMatrixPlots(Matrix *);
 	void add3DData();
 	void change3DData();
 	void change3DData(const QString& colName);
@@ -340,6 +357,8 @@ public slots:
 	Matrix* matrix(const QString& name);
 	Matrix* convertTableToMatrix();
 	Matrix* tableToMatrix(Table* t);
+	Matrix* convertTableToMatrixRegularXYZ();
+	void showBinMatrixDialog();
 	void initMatrix(Matrix* m, const QString& caption);
 	void transposeMatrix();
 	void invertMatrix();
@@ -400,12 +419,12 @@ public slots:
 	void initTable(Table* w, const QString& caption);
 	void customTable(Table* w);
 
+	void importWaveFile();
 	void importASCII();
 	void importASCII(const QStringList& files, int import_mode, const QString& local_column_separator, int local_ignored_lines, bool local_rename_columns,
         bool local_strip_spaces, bool local_simplify_spaces, bool local_import_comments,
         QLocale local_separators, const QString& local_comment_string, bool import_read_only, int endLineChar);
-	void exportAllTables(const QString& sep, bool colNames, bool colComments, bool expSelection);
-	void exportASCII(const QString& tableName, const QString& sep, bool colNames, bool colComments, bool expSelection);
+	void exportAllTables(const QString& dir, const QString& filter, const QString& sep, bool colNames, bool colComments, bool expSelection);
 
 	//! recalculate selected cells of current table
 	void recalculateTable();
@@ -427,6 +446,8 @@ public slots:
 	void plotPie();
 	void plotVerticalBars();
 	void plotHorizontalBars();
+	void plotStackBar();
+	void plotStackColumn();
 	void plotArea();
 	void plotVertSteps();
 	void plotHorSteps();
@@ -464,8 +485,10 @@ public slots:
 	void exportLayer();
 	void exportGraph(const QString& exportFilter = QString::null);
 	void exportAllGraphs();
+	void exportPresentationODF();
 	void exportPDF();
 	void print();
+	void printPreview();
 	void printAllPlots();
 	//@}
 
@@ -576,6 +599,9 @@ public slots:
 	void clearTable();
 	void goToRow();
 	void goToColumn();
+	void moveTableRowUp();
+	void moveTableRowDown();
+	void adjustColumnWidth();
 	//@}
 
 	//! \name Plot Tools
@@ -592,6 +618,7 @@ public slots:
 	void addImage();
 	void zoomIn();
 	void zoomOut();
+	void magnify();
 	void setAutoScale();
 	void showRangeSelectors();
 	void showCursor();
@@ -613,6 +640,7 @@ public slots:
 	void fitMultiPeak(int profile);
 	void fitMultiPeakGauss();
 	void fitMultiPeakLorentz();
+	void fitSlope();
 	//@}
 
 	//! \name Calculus
@@ -625,9 +653,9 @@ public slots:
 	//@}
 
 	void addErrorBars();
-	void defineErrorBars(const QString& name,int type,const QString& percent,int direction);
-	void defineErrorBars(const QString& curveName,const QString& errColumnName, int direction);
-	void movePoints();
+	void defineErrorBars(const QString& name, int type, double percent, int direction);
+	void defineErrorBars(const QString& curveName, const QString& errColumnName, int direction);
+	void movePoints(bool wholeCurve = false);
 	void removePoints();
 
 	//! \name Event Handlers
@@ -674,7 +702,7 @@ public slots:
 	void showTableContextMenu(bool selection);
 	void showWindowContextMenu();
 	void customWindowTitleBarMenu(MdiSubWindow *w, QMenu *menu);
-	void showCurveContextMenu(QwtPlotCurve *);
+	void showCurveContextMenu(QwtPlotItem *);
 	void showCurvePlotDialog();
 	void showCurveWorksheet();
     void showCurveWorksheet(Graph *g, int curveIndex);
@@ -686,7 +714,7 @@ public slots:
 	//! Connected to the context menu signal from lv; it's called when there are no items selected in the list
 	void showListViewPopupMenu(const QPoint &p);
 
-	void showScriptWindow();
+	void showScriptWindow(bool parent = true);
 	void showMoreWindows();
 	void showMarkerPopupMenu();
 	void showHelp();
@@ -838,6 +866,18 @@ public slots:
 	Note* openNote(ApplicationWindow* app, const QStringList &flist);
 	void saveNoteAs();
 	void showNoteLineNumbers(bool show = true);
+	void increaseNoteIndent();
+	void decreaseNoteIndent();
+	void noteFindDialogue();
+	void noteFindNext();
+	void noteFindPrev();
+	void noteReplaceDialogue();
+	void renameCurrentNoteTab();
+	void addNoteTab();
+	void closeNoteTab();
+	void execute();
+	void executeAll();
+	void evaluate();
 	//@}
 
 	//! \name Folders
@@ -1007,6 +1047,7 @@ private slots:
 	//@}
 
 	void analysisMenuAboutToShow();
+	void scriptingMenuAboutToShow();
 	void fileMenuAboutToShow();
 	void editMenuAboutToShow();
 	void matrixMenuAboutToShow();
@@ -1038,8 +1079,14 @@ private slots:
 	void showAllColumns();
 	void closedLastCopiedLayer(){lastCopiedLayer = NULL;};
 
+	void increasePrecision();
+	void decreasePrecision();
+
 // TODO: a lot of this stuff should be private
 public:
+	Graph::LegendDisplayMode d_graph_legend_display;
+	int d_latex_compiler;
+	QString d_latex_compiler_path;
 	//! Last selected filter in open project dialog
     QString d_open_project_filter;
 	//! Default geometry unit to be displayed in the EnrichmentDialog.
@@ -1060,9 +1107,11 @@ public:
 	QString d_translations_folder;
 	//! Flag telling if the application is opening a project file or not
 	bool d_opening_file;
+	//! Flag telling if the application is appending a project file or not
+	bool d_is_appending_file;
     QString customActionsDirPath;
 	bool d_matrix_tool_bar, d_file_tool_bar, d_table_tool_bar, d_column_tool_bar, d_edit_tool_bar;
-	bool d_plot_tool_bar, d_plot3D_tool_bar, d_display_tool_bar, d_format_tool_bar;
+	bool d_plot_tool_bar, d_plot3D_tool_bar, d_display_tool_bar, d_format_tool_bar, d_notes_tool_bar;
 	bool d_backup_files;
 	WindowType d_init_window_type;
 	QRect d_script_win_rect, d_app_rect;
@@ -1071,13 +1120,15 @@ public:
 	QString d_export_col_separator;
 	bool d_export_col_names, d_export_table_selection, d_export_col_comment;
     //! Last selected filter in export image dialog
-    QString d_image_export_filter;
+    QString d_image_export_filter, d_export_ASCII_file_filter;
     double d_scale_fonts_factor;
     bool d_export_transparency;
     int d_export_quality;
     int d_export_vector_resolution, d_export_bitmap_resolution, d_export_size_unit;
     QSizeF d_export_raster_size;
     bool d_export_color;
+    bool d_export_escape_tex_strings;
+    bool d_export_tex_font_sizes;
 	int d_3D_export_text_mode, d_3D_export_sort;
 	//! Locale used to specify the decimal separators in imported ASCII files
 	QLocale d_clipboard_locale;
@@ -1089,6 +1140,7 @@ public:
     QString d_ASCII_file_filter, d_ASCII_comment_string;
 	bool d_ASCII_import_comments, d_ASCII_import_read_only, d_ASCII_import_preview;
 	int d_ASCII_import_mode, d_preview_lines;
+	int d_ASCII_import_first_row_role;
 	//! Specifies if only the Tables/Matrices in the current folder should be displayed in the Add/remove curve dialog.
 	bool d_show_current_folder;
 	bool d_scale_plots_on_print, d_print_cropmarks;
@@ -1210,12 +1262,16 @@ public:
 	QColor d_3D_numbers_color;
 	QColor d_3D_labels_color;
 	QColor d_3D_background_color;
-	QColor d_3D_grid_color;
+	QColor d_3D_grid_color, d_3D_minor_grid_color;
+	bool d_3D_minor_grids, d_3D_major_grids;
+	int d_3D_major_style, d_3D_minor_style;
+	double d_3D_major_width, d_3D_minor_width;
 
     void setPlot3DOptions();
 	//@}
 
 private:
+	bool d_mdi_windows_area;
 	MdiSubWindow *d_active_window;
     TextEditor *d_text_editor;
 	// Flag telling if table values should be automatically recalculated when values in a column are modified.
@@ -1248,7 +1304,7 @@ private:
 	QMdiArea *d_workspace;
 
     QToolBar *fileTools, *plotTools, *tableTools, *columnTools, *plot3DTools, *displayBar, *editTools, *plotMatrixBar;
-	QToolBar *formatToolBar;
+	QToolBar *formatToolBar, *noteTools;
 	QToolButton *btnResults;
 	QWidgetList *hiddenWindows;
 	QLineEdit *info;
@@ -1258,14 +1314,14 @@ private:
 	QMenu *windowsMenu, *foldersMenu, *view, *graph, *fileMenu, *format, *edit, *recent;
 	QMenu *help, *plot2DMenu, *analysisMenu, *multiPeakMenu;
 	QMenu *matrixMenu, *plot3DMenu, *plotDataMenu, *tablesDepend, *scriptingMenu;
-	QMenu *tableMenu, *fillMenu, *normMenu, *newMenu, *exportPlotMenu, *smoothMenu, *filterMenu, *decayMenu;
+	QMenu *tableMenu, *fillMenu, *normMenu, *newMenu, *exportPlotMenu, *smoothMenu, *filterMenu, *decayMenu, *importMenu;
 
 	QAction *actionEditCurveRange, *actionCurveFullRange, *actionShowAllCurves, *actionHideCurve, *actionHideOtherCurves;
 	QAction *actionEditFunction, *actionRemoveCurve, *actionShowCurveWorksheet, *actionShowCurvePlotDialog;
-    QAction *actionNewProject, *actionNewNote, *actionNewTable, *actionNewFunctionPlot;
+    QAction *actionNewProject, *actionAppendProject, *actionNewNote, *actionNewTable, *actionNewFunctionPlot;
     QAction *actionNewSurfacePlot, *actionNewMatrix, *actionNewGraph, *actionNewFolder;
     QAction *actionOpen, *actionLoadImage, *actionSaveProject, *actionSaveProjectAs, *actionImportImage;
-    QAction *actionLoad, *actionUndo, *actionRedo;
+    QAction *actionLoad, *actionUndo, *actionRedo, *actionImportSound;
     QAction *actionCopyWindow, *actionShowAllColumns, *actionHideSelectedColumns;
     QAction *actionCutSelection, *actionCopySelection, *actionPasteSelection, *actionClearSelection;
     QAction *actionShowExplorer, *actionShowLog, *actionAddLayer, *actionShowLayerDialog, *actionAutomaticLayout;
@@ -1275,23 +1331,24 @@ private:
     QAction *actionSwapColumns, *actionMoveColRight, *actionMoveColLeft, *actionMoveColFirst, *actionMoveColLast;
     QAction *actionExportGraph, *actionExportAllGraphs, *actionPrint, *actionPrintAllPlots, *actionShowExportASCIIDialog;
     QAction *actionExportPDF, *actionReadOnlyCol, *actionStemPlot;
-    QAction *actionCloseAllWindows, *actionClearLogInfo, *actionShowPlotWizard, *actionShowConfigureDialog;
+    QAction *actionCloseAllWindows, *actionCloseProject, *actionClearLogInfo, *actionShowPlotWizard, *actionShowConfigureDialog;
     QAction *actionShowCurvesDialog, *actionAddErrorBars, *actionAddFunctionCurve, *actionUnzoom, *actionNewLegend, *actionAddImage, *actionAddText;
     QAction *actionPlotL, *actionPlotP, *actionPlotLP, *actionPlotVerticalDropLines, *actionPlotSpline;
-    QAction *actionPlotVertSteps, *actionPlotHorSteps, *actionPlotVerticalBars;
+    QAction *actionPlotVertSteps, *actionPlotHorSteps, *actionPlotVerticalBars, *actionStackBars, *actionStackColumns;
 	QAction *actionPlotHorizontalBars, *actionPlotArea, *actionPlotPie, *actionPlotVectXYAM, *actionPlotVectXYXY;
     QAction *actionPlotHistogram, *actionPlotStackedHistograms, *actionPlot2VerticalLayers, *actionPlot2HorizontalLayers, *actionPlot4Layers, *actionPlotStackedLayers;
     QAction *actionPlot3DRibbon, *actionPlot3DBars, *actionPlot3DScatter, *actionPlot3DTrajectory;
     QAction *actionPlotDoubleYAxis, *actionAddInsetLayer, *actionAddInsetCurveLayer;
     QAction *actionShowColStatistics, *actionShowRowStatistics, *actionShowIntDialog, *actionIntegrate;
-    QAction *actionDifferentiate, *actionFitLinear, *actionShowFitPolynomDialog;
+    QAction *actionDifferentiate, *actionFitLinear, *actionFitSlope, *actionShowFitPolynomDialog;
     QAction *actionShowExpDecayDialog, *actionShowTwoExpDecayDialog, *actionShowExpDecay3Dialog;
     QAction *actionFitExpGrowth, *actionFitSigmoidal, *actionFitGauss, *actionFitLorentz, *actionShowFitDialog;
     QAction *actionShowAxisDialog, *actionShowTitleDialog;
     QAction *actionShowColumnOptionsDialog, *actionShowColumnValuesDialog, *actionShowColsDialog, *actionShowRowsDialog;
     QAction *actionTableRecalculate, *actionExtractGraphs, *actionExtractLayers;
     QAction *actionAbout, *actionShowHelp, *actionChooseHelpFolder;
-    QAction *actionRename, *actionCloseWindow, *actionConvertTable;
+    QAction *actionRename, *actionCloseWindow;
+    QAction *actionConvertTableDirect, *actionConvertTableBinning, *actionConvertTableRegularXYZ;
     QAction *actionAddColToTable, *actionDeleteLayer, *actionInterpolate;
     QAction *actionResizeActiveWindow, *actionHideActiveWindow;
     QAction *actionShowMoreWindows, *actionPixelLineProfile, *actionIntensityTable;
@@ -1319,7 +1376,7 @@ private:
 	QAction *actionShowScriptWindow, *actionFrequencyCount;
 	QAction *actionAnimate, *actionPerspective, *actionFitFrame, *actionResetRotation;
     QAction *actionDeleteRows, *actionDrawPoints, *actionAddZoomPlot;
-	QAction *btnCursor, *btnSelect, *btnPicker, *btnRemovePoints, *btnMovePoints;
+	QAction *btnCursor, *btnSelect, *btnPicker, *btnRemovePoints, *btnMovePoints, *actionDragCurve;
 	QAction *btnZoomIn, *btnZoomOut, *btnPointer, *btnLine, *btnArrow;
 	QAction *actionFlipMatrixVertically, *actionFlipMatrixHorizontally, *actionRotateMatrix;
 	QAction *actionViewMatrixImage, *actionViewMatrix, *actionExportMatrix, *actionMatrixDefaultScale;
@@ -1334,6 +1391,11 @@ private:
     QAction *wireframe, *hiddenline, *polygon, *filledmesh, *pointstyle, *barstyle, *conestyle, *crossHairStyle;
     QAction *actionShowUndoStack, *actionShowNoteLineNumbers, *actionAddFormula, *actionAddRectangle, *actionAddEllipse;
     QActionGroup *coord, *floorstyle, *grids, *plotstyle, *dataTools;
+    QAction *actionMagnify, *actionFindWindow, *actionWaterfallPlot;
+    QAction *actionMoveRowUp, *actionMoveRowDown, *actionAdjustColumnWidth;
+    QAction *actionPresentationODF, *actionRenameNoteTab, *actionAddNoteTab, *actionCloseNoteTab;
+    QAction *actionIncreaseIndent, *actionDecreaseIndent, *actionFind, *actionFindNext, *actionFindPrev, *actionReplace;
+    QAction *actionIncreasePrecision, *actionDecreasePrecision, *actionPrintPreview;
     QList<QAction *> d_user_actions;
     QUndoView *d_undo_view;
     QList<QMenu *> d_user_menus;
