@@ -66,6 +66,9 @@
 #include <QFontMetrics>
 #include <QFileDialog>
 #include <QFontComboBox>
+#include <QNetworkProxy>
+#include <QCompleter>
+#include <QDirModel>
 
 static const char * notes_32_xpm[] = {
 "32 32 343 2",
@@ -479,6 +482,7 @@ ConfigDialog::ConfigDialog( QWidget* parent, Qt::WFlags fl )
     : QDialog( parent, fl )
 {
 	setAttribute(Qt::WA_DeleteOnClose);
+	setSizeGripEnabled(true);
 
 	ApplicationWindow *app = (ApplicationWindow *)parentWidget();
 	d_3D_title_font = app->d_3D_title_font;
@@ -693,7 +697,13 @@ void ConfigDialog::initPlotsPage()
 	boxMargin->setSingleStep(5);
 	boxMargin->setValue(app->defaultPlotMargin);
 	optionsLayout->addWidget(boxMargin, 5, 1);
-	optionsLayout->setRowStretch(6, 1);
+
+	legendDisplayLabel = new QLabel;
+	optionsLayout->addWidget(legendDisplayLabel, 6, 0);
+	legendDisplayBox = new QComboBox;
+	optionsLayout->addWidget(legendDisplayBox, 6, 1);
+
+	optionsLayout->setRowStretch(7, 1);
 
 	groupBackgroundOptions = new QGroupBox(tr("Background"));
 	optionsTabLayout->addWidget( groupBackgroundOptions );
@@ -903,19 +913,16 @@ void ConfigDialog::initPlots3DPage()
 	middleLayout->addWidget(btnAxes, 0, 0);
 	btnLabels = new ColorButton();
 	btnLabels->setColor(app->d_3D_labels_color);
-	middleLayout->addWidget(btnLabels, 1, 0);
+	middleLayout->addWidget(btnLabels, 0, 1);
 	btnNumbers = new ColorButton();
 	btnNumbers->setColor(app->d_3D_numbers_color);
-	middleLayout->addWidget(btnNumbers, 2, 0);
+	middleLayout->addWidget(btnNumbers, 0, 2);
 	btnMesh = new ColorButton();
 	btnMesh->setColor(app->d_3D_mesh_color);
-	middleLayout->addWidget(btnMesh, 0, 1);
-	btnGrid = new ColorButton();
-	btnGrid->setColor(app->d_3D_grid_color);
-	middleLayout->addWidget(btnGrid, 1, 1);
+	middleLayout->addWidget(btnMesh, 1, 0);
 	btnBackground3D = new ColorButton();
 	btnBackground3D->setColor(app->d_3D_background_color);
-	middleLayout->addWidget(btnBackground3D, 2, 1);
+	middleLayout->addWidget(btnBackground3D, 1, 1);
 
     groupBox3DFonts = new QGroupBox();
 	QGridLayout * fl = new QGridLayout( groupBox3DFonts );
@@ -927,9 +934,64 @@ void ConfigDialog::initPlots3DPage()
 	fl->addWidget( btnNumFnt, 0, 2);
 	fl->setRowStretch(1, 1);
 
+	groupBox3DGrids = new QGroupBox(tr("Grids"));
+	QGridLayout *gl1 = new QGridLayout(groupBox3DGrids);
+
+	boxMajorGrids = new QCheckBox(tr("Ma&jor Grids"));
+	boxMajorGrids->setChecked(app->d_3D_major_grids);
+	connect(boxMajorGrids, SIGNAL(toggled(bool)), this, SLOT(enableMajorGrids(bool)));
+	gl1->addWidget(boxMajorGrids, 0, 1);
+
+	boxMinorGrids = new QCheckBox(tr("Mi&nor Grids"));
+	boxMinorGrids->setChecked(app->d_3D_minor_grids);
+	connect(boxMinorGrids, SIGNAL(toggled(bool)), this, SLOT(enableMinorGrids(bool)));
+	gl1->addWidget(boxMinorGrids, 0, 2);
+
+	label3DGridsColor = new QLabel(tr("Color"));
+    gl1->addWidget(label3DGridsColor, 1, 0);
+
+	btnGrid = new ColorButton();
+	btnGrid->setColor(app->d_3D_grid_color);
+    gl1->addWidget(btnGrid, 1, 1);
+
+    btnGridMinor = new ColorButton();
+    btnGridMinor->setColor(app->d_3D_minor_grid_color);
+    gl1->addWidget(btnGridMinor, 1, 2);
+
+	label3DGridsStyle = new QLabel(tr("Style"));
+    gl1->addWidget(label3DGridsStyle, 2, 0);
+
+	boxMajorGridStyle = new QComboBox();
+	gl1->addWidget(boxMajorGridStyle, 2, 1);
+
+	boxMinorGridStyle = new QComboBox();
+	gl1->addWidget(boxMinorGridStyle, 2, 2);
+
+	label3DGridsWidth = new QLabel(tr("Width"));
+	gl1->addWidget(label3DGridsWidth, 3, 0);
+
+	boxMajorGridWidth = new DoubleSpinBox();
+	boxMajorGridWidth->setLocale(app->locale());
+	boxMajorGridWidth->setMinimum(0.0);
+	boxMajorGridWidth->setValue(app->d_3D_major_width);
+	gl1->addWidget(boxMajorGridWidth, 3, 1);
+
+	boxMinorGridWidth = new DoubleSpinBox();
+	boxMinorGridWidth->setLocale(app->locale());
+	boxMinorGridWidth->setMinimum(0.0);
+	boxMinorGridWidth->setValue(app->d_3D_minor_width);
+	gl1->addWidget(boxMinorGridWidth, 3, 2);
+
+	gl1->setRowStretch(4, 1);
+	gl1->setColumnStretch(3, 1);
+
+	enableMajorGrids(app->d_3D_major_grids);
+	enableMinorGrids(app->d_3D_minor_grids);
+
     QVBoxLayout *vl = new QVBoxLayout();
     vl->addWidget(groupBox3DCol);
     vl->addWidget(groupBox3DFonts);
+    vl->addWidget(groupBox3DGrids);
 
     QHBoxLayout *hb = new QHBoxLayout();
     hb->addWidget(colorMapBox);
@@ -1105,6 +1167,7 @@ void ConfigDialog::initAppPage()
 	appTabWidget->addTab( numericFormatPage, QString() );
 
 	initFileLocationsPage();
+	initProxyPage();
 
 	connect( boxLanguage, SIGNAL( activated(int) ), this, SLOT( switchToLanguage(int) ) );
 	connect( fontsBtn, SIGNAL( clicked() ), this, SLOT( pickApplicationFont() ) );
@@ -1455,6 +1518,11 @@ void ConfigDialog::initFileLocationsPage()
 	ApplicationWindow *app = (ApplicationWindow *)parentWidget();
 	fileLocationsPage = new QWidget();
 
+	QCompleter *completer = new QCompleter(this);
+	completer->setModel(new QDirModel(completer));
+    completer->setModelSorting(QCompleter::CaseSensitivelySortedModel);
+    completer->setCompletionMode(QCompleter::InlineCompletion);
+
 	QGroupBox *gb = new QGroupBox();
 	QGridLayout *gl = new QGridLayout(gb);
 
@@ -1462,7 +1530,8 @@ void ConfigDialog::initFileLocationsPage()
 	gl->addWidget(lblTranslationsPath , 0, 0);
 
 	translationsPathLine = new QLineEdit();
-	translationsPathLine->setText(app->d_translations_folder);
+	translationsPathLine->setText(QDir::toNativeSeparators(app->d_translations_folder));
+	translationsPathLine->setCompleter(completer);
 	gl->addWidget(translationsPathLine, 0, 1);
 
 	QPushButton *browseTranslationsBtn = new QPushButton();
@@ -1473,28 +1542,46 @@ void ConfigDialog::initFileLocationsPage()
 	gl->addWidget(lblHelpPath, 1, 0 );
 
 	QFileInfo hfi(app->helpFilePath);
-	helpPathLine = new QLineEdit(hfi.dir().absolutePath());
+	helpPathLine = new QLineEdit(QDir::toNativeSeparators(hfi.dir().absolutePath()));
+	helpPathLine->setCompleter(completer);
 	gl->addWidget( helpPathLine, 1, 1);
 
 	QPushButton *browseHelpBtn = new QPushButton();
 	browseHelpBtn->setIcon(QIcon(QPixmap(choose_folder_xpm)));
 	gl->addWidget(browseHelpBtn, 1, 2);
-	gl->setRowStretch(2, 1);
+
+	texCompilerLabel = new QLabel(tr("LaTeX Compiler"));
+	gl->addWidget(texCompilerLabel, 2, 0);
+
+	texCompilerPathBox = new QLineEdit(QDir::toNativeSeparators(app->d_latex_compiler_path));
+	texCompilerPathBox->setCompleter(completer);
+	connect(texCompilerPathBox, SIGNAL(editingFinished ()), this, SLOT(validateTexCompiler()));
+
+	gl->addWidget(texCompilerPathBox, 2, 1);
+
+    browseTexCompilerBtn = new QPushButton;
+    browseTexCompilerBtn->setIcon(QIcon(QPixmap(choose_folder_xpm)));
+	connect(browseTexCompilerBtn, SIGNAL(clicked()), this, SLOT(chooseTexCompiler()));
+
+    gl->addWidget(browseTexCompilerBtn, 2, 2);
+	gl->setRowStretch(3, 1);
 
 #ifdef SCRIPTING_PYTHON
 	lblPythonConfigDir = new QLabel(tr("Python Configuration Files"));
-	gl->addWidget(lblPythonConfigDir, 2, 0);
+	gl->addWidget(lblPythonConfigDir, 3, 0);
 
-	pythonConfigDirLine = new QLineEdit(app->d_python_config_folder);
-	gl->addWidget(pythonConfigDirLine, 2, 1);
+	pythonConfigDirLine = new QLineEdit(QDir::toNativeSeparators(app->d_python_config_folder));
+	pythonConfigDirLine->setCompleter(completer);
+	gl->addWidget(pythonConfigDirLine, 3, 1);
 
 	QPushButton *browsePythonConfigBtn = new QPushButton();
 	browsePythonConfigBtn->setIcon(QIcon(QPixmap(choose_folder_xpm)));
 	connect(browsePythonConfigBtn, SIGNAL(clicked()), this, SLOT(choosePythonConfigFolder()));
-	gl->addWidget(browsePythonConfigBtn, 2, 2);
-	gl->setRowStretch(3, 1);
+	gl->addWidget(browsePythonConfigBtn, 3, 2);
+	gl->setRowStretch(4, 1);
 #endif
 
+
 	QVBoxLayout *vl = new QVBoxLayout(fileLocationsPage);
 	vl->addWidget(gb);
 
@@ -1549,6 +1636,14 @@ void ConfigDialog::languageChange()
 	boxAutoscaling->setText(tr("Auto&scaling"));
 	boxAntialiasing->setText(tr("Antia&liasing"));
 
+	legendDisplayLabel->setText(tr("Legend display" ));
+	legendDisplayBox->clear();
+	legendDisplayBox->addItem(tr("Column name"));
+	legendDisplayBox->addItem(tr("Column comment"));
+	legendDisplayBox->addItem(tr("Table name"));
+	legendDisplayBox->addItem(tr("Table legend"));
+	legendDisplayBox->setCurrentIndex(app->d_graph_legend_display);
+
 	groupBackgroundOptions->setTitle(tr("Background"));
 	labelGraphBkgColor->setText(tr("Background Color"));
 	labelGraphBkgOpacity->setText(tr( "Opacity" ));
@@ -1617,6 +1712,7 @@ void ConfigDialog::languageChange()
 	appTabWidget->setTabText(appTabWidget->indexOf(appColors), tr("Colors"));
 	appTabWidget->setTabText(appTabWidget->indexOf(numericFormatPage), tr("Numeric Format"));
 	appTabWidget->setTabText(appTabWidget->indexOf(fileLocationsPage), tr("File Locations"));
+	appTabWidget->setTabText(appTabWidget->indexOf(proxyPage), tr("&Internet Connection"));
 
 	lblLanguage->setText(tr("Language"));
 	lblStyle->setText(tr("Style"));
@@ -1681,6 +1777,13 @@ void ConfigDialog::languageChange()
 	lblPythonConfigDir->setText(tr("Python Configuration Files"));
 #endif
 
+	//proxy tab
+	proxyGroupBox->setTitle(tr("&Proxy"));
+    proxyHostLabel->setText(tr("Host"));
+    proxyPortLabel->setText(tr("Port"));
+    proxyUserLabel->setText(tr("Username"));
+    proxyPasswordLabel->setText(tr("Password"));
+
 	//tables page
 	boxUpdateTableValues->setText(tr("Automatically &Recalculate Column Values"));
 	boxTableComments->setText(tr("&Display Comments in Header"));
@@ -1768,6 +1871,36 @@ void ConfigDialog::languageChange()
 	btnNumFnt->setText( tr( "&Numbers" ) );
 	boxAutoscale3DPlots->setText( tr( "Autosca&ling" ) );
 
+	groupBox3DGrids->setTitle(tr("Grids"));
+	boxMajorGrids->setText(tr("Ma&jor Grids"));
+	boxMinorGrids->setText(tr("Mi&nor Grids"));
+
+	boxMajorGridStyle->clear();
+	boxMajorGridStyle->addItem(tr("Solid"));
+	boxMajorGridStyle->addItem(tr("Dash"));
+	boxMajorGridStyle->addItem(tr("Dot"));
+	boxMajorGridStyle->addItem(tr("Dash Dot"));
+	boxMajorGridStyle->addItem(tr("Dash Dot Dot"));
+	boxMajorGridStyle->addItem(tr("Short Dash"));
+	boxMajorGridStyle->addItem(tr("Short Dot"));
+	boxMajorGridStyle->addItem(tr("Short Dash Dot"));
+	boxMajorGridStyle->setCurrentIndex(app->d_3D_major_style);
+
+	boxMinorGridStyle->clear();
+	boxMinorGridStyle->addItem(tr("Solid"));
+	boxMinorGridStyle->addItem(tr("Dash"));
+	boxMinorGridStyle->addItem(tr("Dot"));
+	boxMinorGridStyle->addItem(tr("Dash Dot"));
+	boxMinorGridStyle->addItem(tr("Dash Dot Dot"));
+	boxMinorGridStyle->addItem(tr("Short Dash"));
+	boxMinorGridStyle->addItem(tr("Short Dot"));
+	boxMinorGridStyle->addItem(tr("Short Dash Dot"));
+	boxMinorGridStyle->setCurrentIndex(app->d_3D_minor_style);
+
+	label3DGridsColor->setText(tr("Color"));
+	label3DGridsWidth->setText(tr("Style"));
+	label3DGridsStyle->setText(tr("Width"));
+
     //Notes page
     labelTabLength->setText(tr("Tab length (pixels)"));
     labelNotesFont->setText(tr("Font"));
@@ -1872,6 +2005,7 @@ void ConfigDialog::apply()
 
 	app->defaultPlotMargin = boxMargin->value();
 	app->d_graph_axes_labels_dist = boxAxesLabelsDist->value();
+	app->d_graph_legend_display = (Graph::LegendDisplayMode)legendDisplayBox->currentIndex();
 	app->setGraphDefaultSettings(boxAutoscaling->isChecked(), boxScaleFonts->isChecked(),
 		boxResize->isChecked(), boxAntialiasing->isChecked());
 	// 2D plots page: curves tab
@@ -2016,6 +2150,11 @@ void ConfigDialog::apply()
 #endif
 	}
 
+	if (generalDialog->currentWidget() == appTabWidget &&
+		appTabWidget->currentWidget() == proxyPage){
+		setApplicationCustomProxy();
+	}
+
 	// general page: confirmations tab
 	app->d_inform_rename_table = boxPromptRenameTables->isChecked();
 	app->confirmCloseFolder = boxFolders->isChecked();
@@ -2043,6 +2182,15 @@ void ConfigDialog::apply()
 	app->d_3D_autoscale = boxAutoscale3DPlots->isChecked();
 	app->setPlot3DOptions();
 
+	app->d_3D_grid_color = btnGrid->color();
+	app->d_3D_minor_grid_color = btnGridMinor->color();
+	app->d_3D_minor_grids = boxMajorGrids->isChecked();
+	app->d_3D_major_grids = boxMinorGrids->isChecked();
+	app->d_3D_major_style = boxMajorGridStyle->currentIndex();
+	app->d_3D_minor_style = boxMinorGridStyle->currentIndex();
+	app->d_3D_major_width = boxMajorGridWidth->value();
+	app->d_3D_minor_width = boxMinorGridWidth->value();
+
 	// fitting page
 	app->fit_output_precision = boxPrecision->value();
 	app->pasteFitResultsToPlot = plotLabelBox->isChecked();
@@ -2293,14 +2441,14 @@ void ConfigDialog::chooseTranslationsFolder()
 	QFileInfo tfi(app->d_translations_folder);
 	QString dir = QFileDialog::getExistingDirectory(this, tr("Choose the location of the QtiPlot translations folder!"),
 #ifdef Q_CC_MSVC
-			tfi.dir().absolutePath(), 0);
+		tfi.dir().absolutePath(), 0);
 #else
-			tfi.dir().absolutePath(), !QFileDialog::ShowDirsOnly);
+		tfi.dir().absolutePath(), !QFileDialog::ShowDirsOnly);
 #endif
 
 	if (!dir.isEmpty()){
-		app->d_translations_folder = dir;
-		translationsPathLine->setText(dir);
+		app->d_translations_folder = QDir::toNativeSeparators(dir);
+		translationsPathLine->setText(app->d_translations_folder);
 		app->createLanguagesList();
 		insertLanguagesList();
 	}
@@ -2312,8 +2460,6 @@ void ConfigDialog::chooseHelpFolder()
 	if (!app)
 		return;
 
-	//app->chooseHelpFolder();
-
 	QFileInfo hfi(app->helpFilePath);
 	QString dir = QFileDialog::getExistingDirectory(this, tr("Choose the location of the QtiPlot help folder!"),
 #ifdef Q_CC_MSVC
@@ -2332,7 +2478,7 @@ void ConfigDialog::chooseHelpFolder()
 			app->helpFilePath = helpFilePath;
 	}
 
-	helpPathLine->setText(QFileInfo(app->helpFilePath).dir().absolutePath());
+	helpPathLine->setText(QDir::toNativeSeparators(QFileInfo(app->helpFilePath).dir().absolutePath()));
 }
 
 #ifdef SCRIPTING_PYTHON
@@ -2345,14 +2491,14 @@ void ConfigDialog::choosePythonConfigFolder()
 	QFileInfo tfi(app->d_python_config_folder);
 	QString dir = QFileDialog::getExistingDirectory(this, tr("Choose the location of the Python configuration files!"),
 #ifdef Q_CC_MSVC
-			tfi.dir().absolutePath(), 0);
+		tfi.dir().absolutePath(), 0);
 #else
-			tfi.dir().absolutePath(), !QFileDialog::ShowDirsOnly);
+		tfi.dir().absolutePath(), !QFileDialog::ShowDirsOnly);
 #endif
 
 	if (!dir.isEmpty()){
-		app->d_python_config_folder = dir;
-		pythonConfigDirLine->setText(dir);
+		app->d_python_config_folder = QDir::toNativeSeparators(dir);
+		pythonConfigDirLine->setText(app->d_python_config_folder);
 
 		if (app->scriptingEnv()->name() == QString("Python"))
 			app->setScriptingLanguage(QString("Python"), true);
@@ -2378,8 +2524,10 @@ void ConfigDialog::rehighlight()
     QList<MdiSubWindow *> windows = app->windowsList();
     foreach(MdiSubWindow *w, windows){
         Note *n = qobject_cast<Note *>(w);
-        if (n)
-			n->editor()->rehighlight();
+        if (n){
+        	for (int i = 0; i < n->tabs(); i++)
+				n->editor(i)->rehighlight();
+        }
     }
 }
 #endif
@@ -2404,7 +2552,7 @@ void ConfigDialog::customizeNotes()
         Note *n = qobject_cast<Note *>(w);
         if (n){
             n->showLineNumbers(app->d_note_line_numbers);
-            n->editor()->setTabStopWidth(app->d_notes_tab_length);
+            n->setTabStopWidth(app->d_notes_tab_length);
             n->setFont(f);
         }
     }
@@ -2447,3 +2595,125 @@ bool ConfigDialog::validFolderPath(const QString& path)
 	return true;
 }
 
+void ConfigDialog::initProxyPage()
+{
+	QNetworkProxy proxy = QNetworkProxy::applicationProxy();
+
+	proxyPage = new QWidget();
+
+	proxyGroupBox = new QGroupBox (tr("&Proxy"));
+	proxyGroupBox->setCheckable(true);
+	proxyGroupBox->setChecked(!proxy.hostName().isEmpty());
+
+	QGridLayout *gl = new QGridLayout(proxyGroupBox);
+
+	proxyHostLabel = new QLabel( tr("Host"));
+    gl->addWidget(proxyHostLabel, 0, 0);
+    proxyHostLine = new QLineEdit(proxy.hostName ());
+    gl->addWidget(proxyHostLine, 0, 1);
+
+	proxyPortLabel = new QLabel( tr("Port"));
+    gl->addWidget(proxyPortLabel, 1, 0);
+    proxyPortBox = new QSpinBox;
+    proxyPortBox->setMaximum(10000000);
+	proxyPortBox->setValue(proxy.port());
+    gl->addWidget(proxyPortBox, 1, 1);
+
+	proxyUserLabel = new QLabel( tr("Username"));
+    gl->addWidget(proxyUserLabel, 2, 0);
+    proxyUserNameLine = new QLineEdit(proxy.user());
+    gl->addWidget(proxyUserNameLine, 2, 1);
+
+	proxyPasswordLabel = new QLabel( tr("Password"));
+    gl->addWidget(proxyPasswordLabel, 3, 0);
+    proxyPasswordLine = new QLineEdit;
+
+    gl->addWidget(proxyPasswordLine, 3, 1);
+
+	gl->setRowStretch(4, 1);
+
+	QVBoxLayout *layout = new QVBoxLayout(proxyPage);
+    layout->addWidget(proxyGroupBox);
+
+	appTabWidget->addTab(proxyPage, tr( "&Internet Connection" ) );
+}
+
+QNetworkProxy ConfigDialog::setApplicationCustomProxy()
+{
+	QNetworkProxy proxy;
+	proxy.setType(QNetworkProxy::NoProxy);
+	if (proxyGroupBox->isChecked())
+		proxy.setHostName(proxyHostLine->text());
+	else
+		proxy.setHostName(QString::null);
+
+	proxy.setPort(proxyPortBox->value());
+	proxy.setUser(proxyUserNameLine->text());
+	proxy.setPassword(proxyPasswordLine->text());
+	QNetworkProxy::setApplicationProxy(proxy);
+	return proxy;
+}
+
+void ConfigDialog::chooseTexCompiler()
+{
+	ApplicationWindow *app = (ApplicationWindow *)parentWidget();
+	if (!app)
+		return;
+
+	QFileInfo tfi(app->d_latex_compiler_path);
+	QString compiler = ApplicationWindow::getFileName(this, tr("Choose the location of the LaTeX compiler!"),
+	app->d_latex_compiler_path, QString(), 0, false);
+
+	if (!compiler.isEmpty()){
+		app->d_latex_compiler_path = QDir::toNativeSeparators(compiler);
+		texCompilerPathBox->setText(app->d_latex_compiler_path);
+	}
+}
+
+bool ConfigDialog::validateTexCompiler()
+{
+	QString path = texCompilerPathBox->text();
+	if (path.isEmpty())
+		return false;
+
+	ApplicationWindow *app = (ApplicationWindow *)parentWidget();
+	QFileInfo fi(path);
+	if (!fi.exists()){
+		QMessageBox::critical(this, tr("QtiPlot - File Not Found!"),
+		tr("The file %1 doesn't exist.<br>Please choose another file!").arg(path));
+		texCompilerPathBox->setText(app->d_latex_compiler_path);
+		return false;
+	}
+
+	if (fi.isDir()){
+		QMessageBox::critical(this, tr("QtiPlot - File Not Found!"),
+		tr("%1 is a folder.<br>Please choose a file!").arg(path));
+		texCompilerPathBox->setText(app->d_latex_compiler_path);
+		return false;
+	}
+
+	if (!fi.isReadable()){
+		QMessageBox::critical(this, tr("QtiPlot"),
+		tr("You don't have read access rights to file %1.<br>Please choose another file!").arg(path));
+		texCompilerPathBox->setText(app->d_latex_compiler_path);
+		return false;
+	}
+
+	app->d_latex_compiler_path = QDir::toNativeSeparators(path);
+	texCompilerPathBox->setText(app->d_latex_compiler_path);
+	return true;
+}
+
+void ConfigDialog::enableMajorGrids(bool on)
+{
+	btnGrid->setEnabled(on);
+	boxMajorGridStyle->setEnabled(on);
+	boxMajorGridWidth->setEnabled(on);
+}
+
+void ConfigDialog::enableMinorGrids(bool on)
+{
+	btnGridMinor->setEnabled(on);
+	boxMinorGridStyle->setEnabled(on);
+	boxMinorGridWidth->setEnabled(on);
+}
