@@ -2,7 +2,7 @@
     File                 : MultiLayer.cpp
     Project              : QtiPlot
     --------------------------------------------------------------------
-    Copyright            : (C) 2006 by Ion Vasilief
+    Copyright            : (C) 2006 - 2009 by Ion Vasilief
     Email (use @ for *)  : ion_vasilief*yahoo.fr
     Description          : Multi layer widget
 
@@ -39,11 +39,22 @@
 #include <QClipboard>
 #include <QTextStream>
 #include <QSvgGenerator>
+#include <QDir>
+
+#include <QPushButton>
+#include <QCheckBox>
+#include <QLayout>
+#include <QLabel>
+#include <QGroupBox>
+#include <QSpinBox>
+#include <QTextDocumentWriter>
 
 #ifdef EMF_OUTPUT
-#include <EmfEngine.h>
+	#include <EmfEngine.h>
 #endif
 
+#include <QTeXEngine.h>
+
 #include <qwt_plot.h>
 #include <qwt_plot_canvas.h>
 #include <qwt_plot_layout.h>
@@ -51,12 +62,14 @@
 #include <qwt_text_label.h>
 #include <qwt_layout_metrics.h>
 
+#include "PlotCurve.h"
 #include "MultiLayer.h"
 #include "LegendWidget.h"
 #include "Spectrogram.h"
 #include "SelectionMoveResizer.h"
 #include <ApplicationWindow.h>
-
+#include <ColorButton.h>
+#include <pixmaps.h>
 #include <gsl/gsl_vector.h>
 
 LayerButton::LayerButton(const QString& text, QWidget* parent)
@@ -68,6 +81,7 @@ LayerButton::LayerButton(const QString& text, QWidget* parent)
 	setOn(true);
 	setMaximumWidth(btn_size);
 	setMaximumHeight(btn_size);
+	setToolTip(tr("Activate layer"));
 }
 
 void LayerButton::mousePressEvent( QMouseEvent *event )
@@ -101,15 +115,42 @@ hor_align(HCenter),
 vert_align(VCenter),
 d_scale_on_print(true),
 d_print_cropmarks(false),
-d_scale_layers(parent->autoResizeLayers)
+d_scale_layers(parent->autoResizeLayers),
+d_waterfall_offset_x(1),
+d_waterfall_offset_y(3),
+d_is_waterfall_plot(false),
+d_side_lines(false),
+d_waterfall_fill_color(QColor())
 {
 	layerButtonsBox = new QHBoxLayout();
+	waterfallBox = new QHBoxLayout();
+	toolbuttonsBox = new QHBoxLayout();
+
+	d_add_layer_btn = new QPushButton();
+	d_add_layer_btn->setToolTip(tr("Add layer"));
+	d_add_layer_btn->setIcon(QIcon(QPixmap(plus_xpm)));
+	d_add_layer_btn->setMaximumWidth(LayerButton::btnSize());
+	d_add_layer_btn->setMaximumHeight(LayerButton::btnSize());
+	connect (d_add_layer_btn, SIGNAL(clicked()), this->applicationWindow(), SLOT(addLayer()));
+	toolbuttonsBox->addWidget(d_add_layer_btn);
+
+	d_remove_layer_btn = new QPushButton();
+	d_remove_layer_btn->setToolTip(tr("Remove active layer"));
+	d_remove_layer_btn->setIcon(QIcon(QPixmap(delete_xpm)));
+	d_remove_layer_btn->setMaximumWidth(LayerButton::btnSize());
+	d_remove_layer_btn->setMaximumHeight(LayerButton::btnSize());
+	connect (d_remove_layer_btn, SIGNAL(clicked()), this, SLOT(confirmRemoveLayer()));
+	toolbuttonsBox->addWidget(d_remove_layer_btn);
+
 #ifdef Q_OS_MAC
 	layerButtonsBox->setSpacing(12);
+	toolbuttonsBox->setSpacing(12);
 #endif
 	QHBoxLayout *hbox = new QHBoxLayout();
 	hbox->addLayout(layerButtonsBox);
 	hbox->addStretch();
+	hbox->addLayout(toolbuttonsBox);
+	hbox->addLayout(waterfallBox);
 
 	d_canvas = new QWidget();
 
@@ -126,7 +167,7 @@ d_scale_layers(parent->autoResizeLayers)
 
     int canvas_width = graph_width + left_margin + right_margin;
     int canvas_height = graph_height + top_margin + bottom_margin;
-    setGeometry(QRect(0, 0, canvas_width, canvas_height + LayerButton::btnSize()));
+	resize(canvas_width, canvas_height + LayerButton::btnSize());
 
     d_canvas->resize(canvas_width, canvas_height);
 	d_canvas->installEventFilter(this);
@@ -140,6 +181,7 @@ d_scale_layers(parent->autoResizeLayers)
 
 	setFocusPolicy(Qt::StrongFocus);
 	setFocus();
+	setAcceptDrops(true);
 }
 
 Graph *MultiLayer::layer(int num)
@@ -169,13 +211,16 @@ Graph* MultiLayer::addLayer(int x, int y, int width, int height)
 {
 	addLayerButton();
 	if (!width && !height){
-		width =	d_canvas->width() - left_margin - right_margin - (d_cols - 1)*colsSpace;
-		height = d_canvas->height() - top_margin - left_margin - (d_rows - 1)*rowsSpace;
+		width =	(int)((d_canvas->width() - left_margin - right_margin - (d_cols - 1)*colsSpace)/(double)d_cols);
+		height = (int)((d_canvas->height() - top_margin - left_margin - (d_rows - 1)*rowsSpace)/(double)d_rows);
 
 		int layers = graphsList.size();
 		x = left_margin + (layers % d_cols)*(width + colsSpace);
 	    y = top_margin + (layers / d_cols)*(height + rowsSpace);
-	}
+	} else if (!width)
+		 width = height;
+	  else if (!height)
+		 height = width;
 
 	Graph* g = new Graph(x, y, width, height, d_canvas);
     g->show();
@@ -194,18 +239,22 @@ void MultiLayer::adjustSize()
 
 void MultiLayer::activateGraph(LayerButton* button)
 {
-	for (int i=0;i<buttonsList.count();i++)
-	{
+	for (int i = 0; i<buttonsList.count(); i++){
 		LayerButton *btn=(LayerButton*)buttonsList.at(i);
 		if (btn->isOn())
 			btn->setOn(false);
 
-		if (btn == button)
-		{
+		if (btn == button){
 			active_graph = (Graph*) graphsList.at(i);
 			active_graph->setFocus();
 			active_graph->raise();//raise layer on top of the layers stack
-			active_graph->raiseEnrichements();
+
+			if (d_layers_selector){
+				delete d_layers_selector;
+				d_layers_selector = new SelectionMoveResizer(active_graph->canvas());
+				connect(d_layers_selector, SIGNAL(targetsChanged()), this, SIGNAL(modifiedPlot()));
+			} else
+				active_graph->raiseEnrichements();
 			button->setOn(true);
 		}
 	}
@@ -222,7 +271,8 @@ void MultiLayer::setActiveLayer(Graph* g)
 	if (d_layers_selector)
 		delete d_layers_selector;
 
-	active_graph->raise();//raise layer on top of the layers stack
+	if (!d_is_waterfall_plot)
+		active_graph->raise();//raise layer on top of the layers stack
 	active_graph->raiseEnrichements();
 
 	for(int i=0; i<graphsList.count(); i++){
@@ -242,16 +292,22 @@ QRect MultiLayer::canvasChildrenRect()
 	QRect r = QRect();
 	foreach (Graph *g, graphsList)
 		r = r.united(g->boundingRect());
-	return r;
+
+	return r.adjusted(0, 0, right_margin, bottom_margin);
 }
 
 void MultiLayer::resizeLayers (QResizeEvent *re)
 {
-	if (!d_scale_layers || applicationWindow()->d_opening_file)
+	if (!d_scale_layers || applicationWindow()->d_opening_file){
+		emit modifiedPlot();
 		return;
+	}
 
-	QSize oldSize = re->oldSize();
 	QSize size = re->size();
+	QSize oldSize = re->oldSize();
+
+	if (size.height() <= 0)
+		size.setHeight(oldSize.height());
 
 	bool scaleLayerFonts = false;
 	if(!oldSize.isValid()){// The old size is invalid when maximizing a window (why?)
@@ -366,6 +422,33 @@ void MultiLayer::setGraphGeometry(int x, int y, int w, int h)
 	emit modifiedPlot();
 }
 
+void MultiLayer::setEqualSizedLayers()
+{
+	int layers = graphsList.size();
+	const QRect rect = d_canvas->geometry();
+
+	int width = qRound((rect.width() - (d_cols - 1)*colsSpace - right_margin - left_margin)/(double)d_cols);
+	int height = qRound((rect.height() - (d_rows - 1)*rowsSpace - top_margin - bottom_margin)/(double)d_rows);
+
+	for(int i = 0; i < layers; i++){
+		Graph *g = graphsList[i];
+
+		int row = i / d_cols;
+		if (row >= d_rows )
+			row = d_rows - 1;
+
+		int col = i % d_cols;
+
+		int x = left_margin + col*(width + colsSpace);
+	    int y = top_margin + row*(height + rowsSpace);
+
+		bool autoscaleFonts = g->autoscaleFonts();//save user settings
+		g->setAutoscaleFonts(false);
+		g->setGeometry(x, y, width, height);
+		g->setAutoscaleFonts(autoscaleFonts);
+	}
+}
+
 QSize MultiLayer::arrangeLayers(bool userSize)
 {
 	int layers = graphsList.size();
@@ -446,7 +529,7 @@ QSize MultiLayer::arrangeLayers(bool userSize)
 	}
 
 	double c_heights = 0.0;
-	for (int i=0; i<d_rows; i++){
+	for (int i = 0; i < d_rows; i++){
 		gsl_vector_set (Y, i, c_heights);
 		c_heights += 1 + gsl_vector_get(maxXTopHeight, i) + gsl_vector_get(maxXBottomHeight, i);
 	}
@@ -458,10 +541,13 @@ QSize MultiLayer::arrangeLayers(bool userSize)
 	}
 
 	if (!userSize){
-		l_canvas_width = int((rect.width()-(d_cols-1)*colsSpace - right_margin - left_margin)/c_widths);
-		l_canvas_height = int((rect.height()-(d_rows-1)*rowsSpace - top_margin - bottom_margin)/c_heights);
+		l_canvas_width = qRound((rect.width()-(d_cols-1)*colsSpace - right_margin - left_margin)/(double)c_widths);
+		l_canvas_height = qRound((rect.height()-(d_rows-1)*rowsSpace - top_margin - bottom_margin)/(double)c_heights);
 	}
 
+	if (l_canvas_width < 50 || l_canvas_height < 50)
+		return QSize();
+
 	QSize size = QSize(l_canvas_width, l_canvas_height);
 
 	for (int i=0; i<layers; i++){
@@ -521,39 +607,20 @@ QSize MultiLayer::arrangeLayers(bool userSize)
 
 void MultiLayer::findBestLayout(int &d_rows, int &d_cols)
 {
-	int NumGraph = graphsList.size();
-	if(NumGraph%2==0) // NumGraph is an even number
-	{
-		if(NumGraph<=2)
-			d_cols=NumGraph/2+1;
-		else if(NumGraph>2)
-			d_cols=NumGraph/2;
+	int layers = graphsList.size();
+	int sqr = (int)ceil(sqrt(layers));
+	d_rows = sqr;
+	d_cols = sqr;
 
-		if(NumGraph<8)
-			d_rows=NumGraph/4+1;
-		if(NumGraph>=8)
-			d_rows=NumGraph/4;
-	}
-	else if(NumGraph%2!=0) // NumGraph is an odd number
-	{
-		int Num=NumGraph+1;
+	if (d_rows*d_cols - layers >= d_rows)
+		d_rows--;
 
-		if(Num<=2)
-			d_cols=1;
-		else if(Num>2)
-			d_cols=Num/2;
-
-		if(Num<8)
-			d_rows=Num/4+1;
-		if(Num>=8)
-			d_rows=Num/4;
-	}
 }
 
-void MultiLayer::arrangeLayers(bool fit, bool userSize)
+bool MultiLayer::arrangeLayers(bool fit, bool userSize)
 {
 	if (graphsList.size() == 0)
-		return;
+		return false;
 
 	QApplication::setOverrideCursor(Qt::waitCursor);
 
@@ -567,11 +634,22 @@ void MultiLayer::arrangeLayers(bool fit, bool userSize)
 	//resize iterations, due to the way Qwt handles the plot layout
 	int iterations = 0;
 	QSize size = arrangeLayers(userSize);
+	if (!size.isValid()){
+		QApplication::restoreOverrideCursor();
+		setEqualSizedLayers();
+		return false;
+	}
+
 	QSize canvas_size = QSize(1,1);
 	while (canvas_size != size && iterations < 10){
 		iterations++;
 		canvas_size = size;
 		size = arrangeLayers(userSize);
+		if (!size.isValid()){
+			QApplication::restoreOverrideCursor();
+			setEqualSizedLayers();
+			return false;
+		}
 	}
 
 	if (userSize){//resize window
@@ -583,6 +661,7 @@ void MultiLayer::arrangeLayers(bool fit, bool userSize)
 
 	emit modifiedPlot();
 	QApplication::restoreOverrideCursor();
+	return true;
 }
 
 void MultiLayer::setCols(int c)
@@ -603,9 +682,12 @@ QPixmap MultiLayer::canvasPixmap(const QSize& size, double scaleFontsFactor)
 		QPixmap pic(d_canvas->size());
 		pic.fill();
 		QPainter p(&pic);
-		foreach (Graph *g, graphsList)
-			g->print(&p, g->geometry());
-
+		QObjectList lst = d_canvas->children();//! this list is sorted according to the stack order
+		foreach (QObject *o, lst){
+			Graph *g = qobject_cast<Graph *>(o);
+			if (g)
+				g->print(&p, g->geometry());
+		}
 		p.end();
 		return pic;
 	}
@@ -619,7 +701,12 @@ QPixmap MultiLayer::canvasPixmap(const QSize& size, double scaleFontsFactor)
 	QPixmap pic(size);
 	pic.fill();
 	QPainter p(&pic);
-	foreach (Graph *g, graphsList){
+	QObjectList lst = d_canvas->children();
+	foreach (QObject *o, lst){
+		Graph *g = qobject_cast<Graph *>(o);
+		if (!g)
+			continue;
+
 		QPoint pos = g->pos();
 		pos = QPoint(int(pos.x()*scaleFactorX), int(pos.y()*scaleFactorY));
 
@@ -647,7 +734,17 @@ void MultiLayer::exportToFile(const QString& fileName)
 	} else if(fileName.contains(".svg")){
 		exportSVG(fileName);
 		return;
-	} else {
+	} else if(fileName.contains(".tex")){
+		exportTeX(fileName);
+		return;
+	}
+#ifdef EMF_OUTPUT
+	 else if(fileName.contains(".emf")){
+		exportEMF(fileName);
+		return;
+	}
+#endif
+	 else {
 		QList<QByteArray> list = QImageWriter::supportedImageFormats();
     	for(int i=0 ; i<list.count() ; i++){
 			if (fileName.contains( "." + list[i].toLower())){
@@ -694,7 +791,65 @@ void MultiLayer::exportImage(const QString& fileName, int quality, bool transpar
 	int dpm = (int)ceil(100.0/2.54*dpi);
 	image.setDotsPerMeterX(dpm);
 	image.setDotsPerMeterY(dpm);
-	image.save(fileName, 0, quality);
+	if (fileName.endsWith(".odf")){
+		QTextDocument *document = new QTextDocument();
+		QTextCursor cursor = QTextCursor(document);
+		cursor.movePosition(QTextCursor::End);
+		cursor.insertText(objectName());
+		cursor.insertBlock();
+		cursor.insertImage(image);
+
+		QTextDocumentWriter writer(fileName);
+		writer.write(document);
+	} else
+		image.save(fileName, 0, quality);
+}
+
+void MultiLayer::exportImage(QTextDocument *document, int quality, bool transparent,
+				int dpi, const QSizeF& customSize, int unit, double fontsFactor)
+{
+	if (!document)
+		return;
+
+	if (!dpi)
+		dpi = logicalDpiX();
+
+	QSize size = QSize();
+	if (customSize.isValid())
+		size = Graph::customPrintSize(customSize, unit, dpi);
+
+	QPixmap pic = canvasPixmap(size, fontsFactor);
+	QImage image = pic.toImage();
+
+	if (transparent){
+		QBitmap mask(size);
+		mask.fill(Qt::color1);
+		QPainter p(&mask);
+		p.setPen(Qt::color0);
+
+		QRgb backgroundPixel = QColor(Qt::white).rgb ();
+		for (int y = 0; y < image.height(); y++){
+			for (int x = 0; x < image.width(); x++){
+				QRgb rgb = image.pixel(x, y);
+				if (rgb == backgroundPixel) // we want the frame transparent
+					p.drawPoint(x, y);
+			}
+		}
+		p.end();
+		pic.setMask(mask);
+		image = pic.toImage();
+	}
+
+	int dpm = (int)ceil(100.0/2.54*dpi);
+	image.setDotsPerMeterX(dpm);
+	image.setDotsPerMeterY(dpm);
+
+	QTextCursor cursor = QTextCursor(document);
+	cursor.movePosition(QTextCursor::End);
+	cursor.insertBlock();
+	cursor.insertText(objectName());
+	cursor.insertBlock();
+	cursor.insertImage(image);
 }
 
 void MultiLayer::exportPDF(const QString& fname)
@@ -734,7 +889,11 @@ void MultiLayer::exportVector(const QString& fileName, int res, bool color,
 			printer.setResolution(res);
 		printer.setPaperSize (QSizeF(size), QPrinter::DevicePixel);
 		QPainter paint(&printer);
-		foreach (Graph *g, graphsList){
+		QObjectList lst = d_canvas->children();
+		foreach (QObject *o, lst){
+			Graph *g = qobject_cast<Graph *>(o);
+			if (!g)
+				continue;
 			QRect r = g->geometry();
 			double wfactor = (double)size.width()/(double)d_canvas->width();
 			double hfactor = (double)size.height()/(double)d_canvas->height();
@@ -752,50 +911,127 @@ void MultiLayer::exportVector(const QString& fileName, int res, bool color,
 		double wfactor = (double)res/(double)logicalDpiX();
 		double hfactor = (double)res/(double)logicalDpiY();
 		printer.setResolution(res);
-		printer.setPaperSize (QSizeF(d_canvas->width()*wfactor*1.05, d_canvas->height()*hfactor), QPrinter::DevicePixel);
+		QRect cr = canvasChildrenRect();
+		printer.setPaperSize (QSizeF(cr.width()*wfactor*1.05, cr.height()*hfactor), QPrinter::DevicePixel);
 		QPainter paint(&printer);
-		foreach (Graph *g, graphsList){
+		QObjectList lst = d_canvas->children();
+		foreach (QObject *o, lst){
+			Graph *g = qobject_cast<Graph *>(o);
+			if (!g)
+				continue;
 			QRect r = g->geometry();
 			r.setSize(QSize(int(r.width()*wfactor), int(r.height()*hfactor)));
 			r.moveTo(int(r.x()*wfactor), int(r.y()*hfactor));
         	g->print(&paint, r);
 		}
 	} else {
-    	printer.setPaperSize(QSizeF(d_canvas->width(), d_canvas->height()), QPrinter::DevicePixel);
+		QRect cr = canvasChildrenRect();
+    	printer.setPaperSize(QSizeF(cr.width(), cr.height()), QPrinter::DevicePixel);
 		QPainter paint(&printer);
-    	foreach (Graph *g, graphsList)
-        	g->print(&paint, g->geometry());
+    	QObjectList lst = d_canvas->children();
+		foreach (QObject *o, lst){
+			Graph *g = qobject_cast<Graph *>(o);
+			if (g)
+				g->print(&paint, g->geometry());
+		}
 	}
 }
 
-void MultiLayer::exportSVG(const QString& fname)
+void MultiLayer::draw(QPaintDevice *device, const QSizeF& customSize, int unit, int res, double fontsFactor)
 {
-	QSvgGenerator generator;
-	generator.setFileName(fname);
-	generator.setSize(d_canvas->size());
-	generator.setResolution(96);
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-	QPainter p(&generator);
-	foreach (Graph *g, graphsList)
-		g->print(&p, g->geometry());
+	QPainter paint(device);
 
-	p.end();
+	if (customSize.isValid()){
+		QSize size = Graph::customPrintSize(customSize, unit, res);
+		QObjectList lst = d_canvas->children();
+		foreach (QObject *o, lst){
+			Graph *g = qobject_cast<Graph *>(o);
+			if (!g)
+				continue;
+			QRect r = g->geometry();
+			double wfactor = (double)size.width()/(double)d_canvas->width();
+			double hfactor = (double)size.height()/(double)d_canvas->height();
+			r.setSize(QSize(int(r.width()*wfactor), int(r.height()*hfactor)));
+			r.moveTo(int(r.x()*wfactor), int(r.y()*hfactor));
+
+			if (fontsFactor == 0.0)
+				fontsFactor = Graph::customPrintSize(customSize, unit, logicalDpiX()).height()/(double)height();
+
+			g->scaleFonts(fontsFactor);
+        	g->print(&paint, r);
+        	g->scaleFonts(1.0/fontsFactor);
+		}
+	} else {
+    	QObjectList lst = d_canvas->children();
+		foreach (QObject *o, lst){
+			Graph *g = qobject_cast<Graph *>(o);
+			if (g)
+				g->print(&paint, g->geometry());
+		}
+	}
+	paint.end();
+	QApplication::restoreOverrideCursor();
+}
+
+void MultiLayer::exportSVG(const QString& fname, const QSizeF& customSize, int unit, double fontsFactor)
+{
+	int res = 96;
+
+	QSvgGenerator svg;
+	svg.setFileName(fname);
+	svg.setSize(canvasChildrenRect().size());
+	svg.setResolution(res);
+
+	if (customSize.isValid()){
+		QSize size = Graph::customPrintSize(customSize, unit, res);
+		svg.setSize(size);
+	}
+
+	draw(&svg, customSize, unit, res, fontsFactor);
 }
 
 #ifdef EMF_OUTPUT
-void MultiLayer::exportEMF(const QString& fname)
+void MultiLayer::exportEMF(const QString& fname, const QSizeF& customSize, int unit, double fontsFactor)
 {
-	EmfPaintDevice emf(d_canvas->size(), fname);
-	QPainter paint;
-	paint.begin(&emf);
+	int res = logicalDpiX();
+	QSize size = canvasChildrenRect().size();
+	if (customSize.isValid())
+		size = Graph::customPrintSize(customSize, unit, res);
 
-	foreach (Graph *g, graphsList)
-		g->print(&paint, g->geometry());
-
-	paint.end();
+	EmfPaintDevice emf(size, fname);
+	draw(&emf, customSize, unit, res, fontsFactor);
 }
 #endif
 
+void MultiLayer::exportTeX(const QString& fname, bool color, bool escapeStrings, bool fontSizes, const QSizeF& customSize, int unit, double fontsFactor)
+{
+	int res = logicalDpiX();
+	QSize size = canvasChildrenRect().size();
+	if (customSize.isValid())
+		size = Graph::customPrintSize(customSize, unit, res);
+
+	QTeXPaintDevice tex(fname, size);
+	tex.setEscapeTextMode(false);
+	tex.exportFontSizes(fontSizes);
+	if (!color)
+		tex.setColorMode(QPrinter::GrayScale);
+
+	foreach (Graph* g, graphsList){
+		g->setTeXExportingMode();
+		g->setEscapeTeXStringsMode(escapeStrings);
+	}
+
+	if (!fontSizes)
+		fontsFactor = 1.0;
+
+	draw(&tex, customSize, unit, res, fontsFactor);
+
+	foreach (Graph* g, graphsList)
+		g->setTeXExportingMode(false);
+}
+
 void MultiLayer::copyAllLayers()
 {
 	bool selectionOn = false;
@@ -804,12 +1040,29 @@ void MultiLayer::copyAllLayers()
 		selectionOn = true;
 	}
 
-	foreach (QWidget* g, graphsList)
-		((Graph *)g)->deselectMarker();
+	foreach (Graph* g, graphsList)
+		g->deselectMarker();
 
+#ifdef EMF_OUTPUT
+	if (OpenClipboard(0)){
+		EmptyClipboard();
+
+		QString path = QDir::tempPath();
+		QString name = path + "/" + "qtiplot_clipboard.emf";
+		name = QDir::cleanPath(name);
+
+		exportEMF(name);
+		HENHMETAFILE handle = GetEnhMetaFile(name.toStdWString().c_str());
+
+		SetClipboardData(CF_ENHMETAFILE, handle);
+		CloseClipboard();
+		QFile::remove(name);
+	}
+#else
 	QPixmap pic = canvasPixmap();
 	QImage image = pic.convertToImage();
 	QApplication::clipboard()->setImage(image);
+#endif
 
 	if (selectionOn)
 		d_layers_selector->show();
@@ -858,6 +1111,16 @@ void MultiLayer::print()
 	}
 }
 
+void MultiLayer::print(QPrinter *printer)
+{
+	if (!printer)
+		return;
+
+	QPainter paint(printer);
+	printAllLayers(&paint);
+	paint.end();
+}
+
 void MultiLayer::printAllLayers(QPainter *painter)
 {
 	if (!painter)
@@ -878,12 +1141,12 @@ void MultiLayer::printAllLayers(QPainter *painter)
 			cr.moveTo(QPoint(margin + int(cr.x()*scaleFactorX),
 							 margin + int(cr.y()*scaleFactorY)));
 			cr.setWidth(int(cr.width()*scaleFactorX));
-			cr.setHeight(int(cr.height()*scaleFactorX));
+			cr.setHeight(int(cr.height()*scaleFactorY));
         }
 
 		foreach (Graph *g, graphsList){
 			QPoint pos = g->pos();
-			pos=QPoint(margin + int(pos.x()*scaleFactorX), margin + int(pos.y()*scaleFactorY));
+			pos = QPoint(margin + int(pos.x()*scaleFactorX), margin + int(pos.y()*scaleFactorY));
 
 			int width = int(g->frameGeometry().width()*scaleFactorX);
 			int height = int(g->frameGeometry().height()*scaleFactorY);
@@ -953,7 +1216,7 @@ void MultiLayer::connectLayer(Graph *g)
 	connect (g,SIGNAL(axisDblClicked(int)),this,SIGNAL(showScaleDialog(int)));
 	connect (g,SIGNAL(showAxisTitleDialog()),this,SIGNAL(showAxisTitleDialog()));
 		connect (g,SIGNAL(showMarkerPopupMenu()),this,SIGNAL(showMarkerPopupMenu()));
-	connect (g,SIGNAL(showCurveContextMenu(QwtPlotCurve *)),this,SIGNAL(showCurveContextMenu(QwtPlotCurve *)));
+	connect (g,SIGNAL(showCurveContextMenu(QwtPlotItem *)),this,SIGNAL(showCurveContextMenu(QwtPlotItem *)));
 	connect (g,SIGNAL(cursorInfo(const QString&)),this,SIGNAL(cursorInfo(const QString&)));
 	connect (g,SIGNAL(viewTitleDialog()),this,SIGNAL(viewTitleDialog()));
 	connect (g,SIGNAL(modifiedGraph()),this,SIGNAL(modifiedPlot()));
@@ -974,7 +1237,10 @@ bool MultiLayer::eventFilter(QObject *object, QEvent *e)
 	    if (me->button() == Qt::RightButton)
             return QMdiSubWindow::eventFilter(object, e);
 
-        QPoint pos = d_canvas->mapFromParent(me->pos());
+		if (d_is_waterfall_plot)
+			return true;
+
+		QPoint pos = d_canvas->mapFromParent(me->pos());
         // iterate backwards, so layers on top are preferred for selection
         QList<Graph*>::iterator i = graphsList.end();
         while (i != graphsList.begin()) {
@@ -987,31 +1253,24 @@ bool MultiLayer::eventFilter(QObject *object, QEvent *e)
             if (igeo.contains(pos)) {
                 if (me->modifiers() & Qt::ShiftModifier) {
                     if (d_layers_selector)
-                        d_layers_selector->add(*i);
+                        d_layers_selector->add((*i)->canvas());
                     else {
-                        d_layers_selector = new SelectionMoveResizer(*i);
+                        d_layers_selector = new SelectionMoveResizer((*i)->canvas());
                         connect(d_layers_selector, SIGNAL(targetsChanged()), this, SIGNAL(modifiedPlot()));
                     }
                 } else {
                     setActiveLayer(g);
                     active_graph->raise();
                     if (!d_layers_selector) {
-                        d_layers_selector = new SelectionMoveResizer(*i);
+                        d_layers_selector = new SelectionMoveResizer((*i)->canvas());
                         connect(d_layers_selector, SIGNAL(targetsChanged()), this, SIGNAL(modifiedPlot()));
                     }
                 }
                 return true;
             }
         }
-        if (d_layers_selector){
-			foreach(Graph *g, graphsList){
-				if (d_layers_selector->contains((QWidget *)g)){
-					g->raiseEnrichements();
-					break;
-				}
-			}
+        if (d_layers_selector)
             delete d_layers_selector;
-		}
 	}
 
 	return MdiSubWindow::eventFilter(object, e);
@@ -1019,6 +1278,11 @@ bool MultiLayer::eventFilter(QObject *object, QEvent *e)
 
 void MultiLayer::keyPressEvent(QKeyEvent * e)
 {
+	if (e->key() == Qt::Key_Escape && d_layers_selector){
+		delete d_layers_selector;
+		return;
+	}
+
 	if (e->key() == Qt::Key_F12){
 		if (d_layers_selector)
 			delete d_layers_selector;
@@ -1133,10 +1397,15 @@ void MultiLayer::save(const QString &fn, const QString &geometry, bool saveAsTem
 	t << "LayerCanvasSize\t"+QString::number(l_canvas_width)+"\t"+QString::number(l_canvas_height)+"\n";
 	t << "Alignement\t"+QString::number(hor_align)+"\t"+QString::number(vert_align)+"\n";
 
-	for (int i=0; i<(int)graphsList.count(); i++){
-		Graph* ag=(Graph*)graphsList.at(i);
-		t << ag->saveToString(saveAsTemplate);
+	foreach (Graph *g, graphsList)
+		t << g->saveToString(saveAsTemplate);
+
+	if (d_is_waterfall_plot){
+		t << "<waterfall>" + QString::number(d_waterfall_offset_x) + ",";
+		t << QString::number(d_waterfall_offset_y) + ",";
+		t << QString::number(d_side_lines) + "</waterfall>\n";
 	}
+
 	t << "</multiLayer>\n";
 }
 
@@ -1233,12 +1502,24 @@ void MultiLayer::copy(MultiLayer* ml)
 	setAlignement(ml->horizontalAlignement(), ml->verticalAlignement());
 	setMargins(ml->leftMargin(), ml->rightMargin(), ml->topMargin(), ml->bottomMargin());
 
-	QList<Graph *> layers = ml->layersList();
-	foreach(Graph *g, layers){
+	QObjectList lst = ml->canvas()->children();
+	foreach (QObject *o, lst){
+		Graph *g = qobject_cast<Graph *>(o);
+		if (!g)
+			continue;
 		Graph* g2 = addLayer(g->pos().x(), g->pos().y(), g->width(), g->height());
+		g2->setAutoscaleFonts(false);
 		g2->copy(g);
 		g2->setAutoscaleFonts(g->autoscaleFonts());
 	}
+
+	if (ml->isWaterfallPlot()){
+		d_waterfall_offset_x = ml->waterfallXOffset();
+		d_waterfall_offset_y = ml->waterfallYOffset();
+		createWaterfallBox();
+		setWaterfallSideLines(ml->sideLinesEnabled());
+	}
+
 	show();
 }
 
@@ -1296,6 +1577,372 @@ bool MultiLayer::hasSelectedLayers()
     return false;
 }
 
+void MultiLayer::setWaterfallLayout(bool on)
+{
+	if (graphsList.isEmpty())
+		return;
+
+	d_is_waterfall_plot = on;
+
+	if (on){
+		createWaterfallBox();
+		updateWaterfallLayout();
+	} else {
+		for (int i = 0; i < waterfallBox->count(); i++){
+			QLayoutItem *item = waterfallBox->itemAt(i);
+			if (item){
+				waterfallBox->removeItem(item);
+				delete item;
+			}
+		}
+	}
+}
+
+void MultiLayer::createWaterfallBox()
+{
+	QPushButton *btn = new QPushButton(tr("Offset Amount..."));
+	connect (btn, SIGNAL(clicked()), this, SLOT(showWaterfallOffsetDialog()));
+
+	waterfallBox->addWidget(btn);
+	btn = new QPushButton(tr("Reverse Order"));
+	connect (btn, SIGNAL(clicked()), this, SLOT(reverseWaterfallOrder()));
+
+	waterfallBox->addWidget(btn);
+	btn = new QPushButton(tr("Fill Area..."));
+	connect (btn, SIGNAL(clicked()), this, SLOT(showWaterfallFillDialog()));
+	waterfallBox->addWidget(btn);
+
+	d_is_waterfall_plot = true;
+
+	foreach(LayerButton *btn, buttonsList)
+		btn->hide();
+
+	d_add_layer_btn->hide();
+	d_remove_layer_btn->hide();
+
+	Graph *first = graphsList.last();
+	if (first)
+		connect(first, SIGNAL(axisDivChanged(Graph *, int)), this, SLOT(updateWaterfallScales(Graph *, int)));
+}
+
+void MultiLayer::updateWaterfallScales(Graph *g, int axis)
+{
+	QwtScaleDiv *scDiv = g->axisScaleDiv(axis);
+	foreach(Graph *l, graphsList){
+		if (l == g)
+			continue;
+
+		l->setAxisScaleDiv(axis, *scDiv);
+		l->replot();
+	}
+}
+
+void MultiLayer::updateWaterfallLayout()
+{
+	if (!d_is_waterfall_plot || graphsList.isEmpty())
+		return;
+
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+	int layers = graphsList.size();
+	int aux = layers - 1;
+	Graph *first = graphsList[aux];
+	bool scaleFonts = first->autoscaleFonts();
+	first->setAutoscaleFonts(false);
+	first->enableAxis(QwtPlot::xBottom, true);
+	first->enableAxis(QwtPlot::yLeft, true);
+	first->enableAxis(QwtPlot::xTop, false);
+	first->enableAxis(QwtPlot::yRight, false);
+
+	if (!first->isVisible())
+		first->show();
+
+	int dx = qRound(d_waterfall_offset_x*d_canvas->width()/100.0);
+	int dy = qRound(d_waterfall_offset_y*d_canvas->height()/100.0);
+	first->resize(QSize(d_canvas->width() - aux*dx - left_margin - right_margin,
+			d_canvas->height() - aux*dy - top_margin - bottom_margin));
+	first->move(first->x(), first->canvas()->y() + aux*dy);
+
+	QSize size = first->canvas()->size();
+	QPoint pos = QPoint(first->canvas()->x() + left_margin,
+						first->canvas()->y() + top_margin);
+
+	QColor c = Qt::white;
+	c.setAlpha(0);
+	first->setBackgroundColor(c);
+	first->setCanvasBackground(c);
+
+	QwtScaleDiv *xScaleDiv = first->axisScaleDiv(QwtPlot::xBottom);
+	QwtScaleDiv *yScaleDiv = first->axisScaleDiv(QwtPlot::yLeft);
+
+	QObjectList lst = d_canvas->children();
+	for (int i = 1; i < layers; i++){
+		Graph *g = qobject_cast<Graph *>(lst[aux - i]);
+		if (!g)
+			continue;
+
+		g->setBackgroundColor(c);
+		g->setCanvasBackground(c);
+
+		for (int j = 0; j < QwtPlot::axisCnt; j++)
+			g->enableAxis(j, false);
+
+		g->move(pos.x() + i*dx, pos.y() + (aux - i)*dy);
+		g->resize(size);
+
+		if (!g->isVisible())
+			g->show();
+
+		g->setAxisScaleDiv(QwtPlot::xBottom, *xScaleDiv);
+		g->setAxisScaleDiv(QwtPlot::yLeft, *yScaleDiv);
+	}
+	first->setAutoscaleFonts(scaleFonts);
+
+	foreach(Graph *g, graphsList)
+		g->updateMarkersBoundingRect();
+	QApplication::restoreOverrideCursor();
+}
+
+void MultiLayer::showWaterfallOffsetDialog()
+{
+	QDialog *offsetDialog = new QDialog(this);
+	offsetDialog->setWindowTitle(tr("Offset Dialog"));
+
+	QGroupBox *gb1 = new QGroupBox();
+	QGridLayout *hl1 = new QGridLayout(gb1);
+
+	hl1->addWidget(new QLabel(tr("Total Y Offset (%)")), 0, 0);
+	QSpinBox *yOffsetBox = new QSpinBox();
+	yOffsetBox->setValue(d_waterfall_offset_y);
+	yOffsetBox->setRange(0, 100000);
+	hl1->addWidget(yOffsetBox, 0, 1);
+
+	hl1->addWidget(new QLabel(tr("Total X Offset (%)")), 1, 0);
+	QSpinBox *xOffsetBox = new QSpinBox();
+	xOffsetBox->setValue(d_waterfall_offset_x);
+	xOffsetBox->setRange(0, 100000);
+	hl1->addWidget(xOffsetBox, 1, 1);
+	hl1->setRowStretch(2, 1);
+
+	connect(yOffsetBox, SIGNAL(valueChanged(int)), this, SLOT(changeWaterfallYOffset(int)));
+	connect(xOffsetBox, SIGNAL(valueChanged(int)), this, SLOT(changeWaterfallXOffset(int)));
+
+	QPushButton *applyBtn = new QPushButton(tr("&Apply"));
+	connect(applyBtn, SIGNAL(clicked()), this, SLOT(updateWaterfallLayout()));
+
+	QPushButton *closeBtn = new QPushButton(tr("&Close"));
+	connect(closeBtn, SIGNAL(clicked()), offsetDialog, SLOT(reject()));
+
+	QHBoxLayout *hl2 = new QHBoxLayout();
+	hl2->addStretch();
+	hl2->addWidget(applyBtn);
+	hl2->addWidget(closeBtn);
+
+	QVBoxLayout *vl = new QVBoxLayout(offsetDialog);
+	vl->addWidget(gb1);
+	vl->addLayout(hl2);
+	offsetDialog->exec();
+}
+
+void MultiLayer::changeWaterfallXOffset(int offset)
+{
+	if (offset == d_waterfall_offset_x)
+		return;
+
+	d_waterfall_offset_x = offset;
+	updateWaterfallLayout();
+	emit modifiedWindow(this);
+}
+
+void MultiLayer::setWaterfallOffset(int x, int y, bool update)
+{
+	d_waterfall_offset_x = x;
+	d_waterfall_offset_y = y;
+
+	if (update){
+		updateWaterfallLayout();
+		emit modifiedWindow(this);
+	}
+}
+
+void MultiLayer::changeWaterfallYOffset(int offset)
+{
+	if (offset == d_waterfall_offset_y)
+		return;
+
+	d_waterfall_offset_y = offset;
+	updateWaterfallLayout();
+	emit modifiedWindow(this);
+}
+
+void MultiLayer::reverseWaterfallOrder()
+{
+	int layers = graphsList.size();
+	if (layers < 2)
+		return;
+
+	QList<QwtPlotItem *> lst;
+	foreach(Graph *g, graphsList){
+		QwtPlotItem *c = g->curve(0);
+		if (c){
+			lst << c;
+			g->removeCurve(c);
+		}
+	}
+
+	if (lst.isEmpty())
+		return;
+
+	for (int i = 0; i < layers; i++){
+		Graph *g = graphsList[i];
+		if (!g)
+			continue;
+
+		int index = layers - i - 1;
+		if (index >= 0 && index < lst.size())
+			g->insertCurve(lst[index]);
+
+		g->replot();
+	}
+	emit modifiedWindow(this);
+}
+
+void MultiLayer::showWaterfallFillDialog()
+{
+	QDialog *waterfallFillDialog = new QDialog(this);
+	waterfallFillDialog->setWindowTitle(tr("Fill Curves"));
+
+	QGroupBox *gb1 = new QGroupBox(tr("Enable Fill"));
+	gb1->setCheckable(true);
+
+	QGridLayout *hl1 = new QGridLayout(gb1);
+	hl1->addWidget(new QLabel(tr("Fill with Color")), 0, 0);
+	ColorButton *fillColorBox = new ColorButton();
+	hl1->addWidget(fillColorBox, 0, 1);
+
+	QCheckBox *sideLinesBox = new QCheckBox(tr("Side Lines"));
+	sideLinesBox->setChecked(d_side_lines);
+	hl1->addWidget(sideLinesBox, 1, 0);
+	hl1->setRowStretch(2, 1);
+
+	if (active_graph){
+		QwtPlotCurve *cv = (QwtPlotCurve *)active_graph->curve(0);
+		if (cv){
+			d_waterfall_fill_color = cv->brush().color();
+			fillColorBox->setColor(d_waterfall_fill_color);
+			gb1->setChecked(cv->brush().style() != Qt::NoBrush);
+		}
+	}
+
+	connect(gb1, SIGNAL(toggled(bool)), this, SLOT(updateWaterfallFill(bool)));
+	connect(fillColorBox, SIGNAL(colorChanged(const QColor&)),
+			this, SLOT(setWaterfallFillColor(const QColor&)));
+	connect(sideLinesBox, SIGNAL(toggled(bool)), this, SLOT(setWaterfallSideLines(bool)));
+
+	QPushButton *closeBtn = new QPushButton(tr("&Close"));
+	connect(closeBtn, SIGNAL(clicked()), waterfallFillDialog, SLOT(reject()));
+
+	QHBoxLayout *hl2 = new QHBoxLayout();
+	hl2->addStretch();
+	hl2->addWidget(closeBtn);
+
+	QVBoxLayout *vl = new QVBoxLayout(waterfallFillDialog);
+	vl->addWidget(gb1);
+	vl->addLayout(hl2);
+	waterfallFillDialog->exec();
+}
+
+void MultiLayer::setWaterfallSideLines(bool on)
+{
+	if (d_side_lines == on)
+		return;
+
+	d_side_lines = on;
+
+	QObjectList lst = d_canvas->children();
+	foreach (QObject *o, lst){
+		Graph *g = qobject_cast<Graph *>(o);
+		if (!g)
+			continue;
+		PlotCurve *cv = (PlotCurve *)g->curve(0);
+		if (cv){
+			cv->enableSideLines(on);
+			g->replot();
+		}
+	}
+	emit modifiedWindow(this);
+}
+
+void MultiLayer::setWaterfallFillColor(const QColor& c)
+{
+	if (d_waterfall_fill_color == c)
+		return;
+
+	d_waterfall_fill_color = c;
+
+	QObjectList lst = d_canvas->children();
+	foreach (QObject *o, lst){
+		Graph *g = qobject_cast<Graph *>(o);
+		if (!g)
+			continue;
+		QwtPlotCurve *cv = (QwtPlotCurve *)g->curve(0);
+		if (cv){
+			cv->setBrush(QBrush(c));
+			g->replot();
+		}
+	}
+	emit modifiedWindow(this);
+}
+
+void MultiLayer::updateWaterfallFill(bool on)
+{
+	QObjectList lst = d_canvas->children();
+	foreach (QObject *o, lst){
+		Graph *g = qobject_cast<Graph *>(o);
+		if (!g)
+			continue;
+		PlotCurve *cv = (PlotCurve *)g->curve(0);
+		if (cv){
+			if (on)
+				cv->setBrush(QBrush(d_waterfall_fill_color));
+			else
+				cv->setBrush(QBrush());
+
+			cv->enableSideLines(d_side_lines);
+			g->replot();
+		}
+	}
+	emit modifiedWindow(this);
+}
+
+void MultiLayer::dragEnterEvent( QDragEnterEvent* e )
+{
+	Graph *g = qobject_cast<Graph*>(e->source());
+	if (!g || g->multiLayer() == this)
+		return;
+
+	if (e->mimeData()->hasFormat("text/plain"))
+		e->acceptProposedAction();
+}
+
+void MultiLayer::dropEvent(QDropEvent* event)
+{
+	Graph *g = qobject_cast<Graph*>(event->source());
+	if (!g)
+		return;
+
+	if (g->multiLayer() == this)
+		return;
+
+	QStringList lst = event->mimeData()->text().split(";");
+
+	QPoint pos = d_canvas->mapFromGlobal(QCursor::pos());
+	pos = QPoint(pos.x() - lst[0].toInt(), pos.y() - lst[1].toInt());
+	Graph *clone = addLayer(pos.x(), pos.y(), g->width(), g->height());
+	if (clone)
+		clone->copy(g);
+}
+
 MultiLayer::~MultiLayer()
 {
 	if(d_layers_selector)
