@@ -35,21 +35,41 @@
 #include <PatternBox.h>
 
 #include <QDateTime>
-#include <QMessageBox>
 #include <QPainter>
 #include <qwt_symbol.h>
+#include <qwt_painter.h>
+#include <qwt_curve_fitter.h>
 
 PlotCurve::PlotCurve(const QString& name): QwtPlotCurve(name),
 d_type(0),
 d_plot_style(0),
 d_x_offset(0.0),
-d_y_offset(0.0)
+d_y_offset(0.0),
+d_side_lines(false),
+d_skip_symbols(1)
 {
     setPaintAttribute(PaintFiltered);
     setPaintAttribute(ClipPolygons);
 	setCurveAttribute(QwtPlotCurve::Fitted, false);
 }
 
+QwtDoubleRect PlotCurve::boundingRect() const
+{
+	QwtDoubleRect r = QwtPlotCurve::boundingRect();
+
+	double percent = 0.01;
+
+	double dw = percent*fabs(r.right() - r.left());
+	r.setLeft(r.left() - dw);
+	r.setRight(r.right() + dw);
+
+	double dh = percent*fabs(r.top() - r.bottom());
+	r.setBottom(r.bottom() + dh);
+	r.setTop(r.top() - dh);
+
+	return r;
+}
+
 QString PlotCurve::saveCurveLayout()
 {
 	QString s = "<Style>" + QString::number(d_plot_style) + "</Style>\n";
@@ -106,8 +126,8 @@ QString PlotCurve::saveCurveLayout()
 void PlotCurve::restoreCurveLayout(const QStringList& lst)
 {
 	QStringList::const_iterator line = lst.begin();
-	for (line++; line != lst.end(); line++){
-        QString s = *line;
+	for (line = lst.begin(); line != lst.end(); line++){
+        QString s = (*line).stripWhiteSpace();
         if (s == "<Pen>"){
 			QPen pen;
 			pen.setCosmetic(true);
@@ -174,6 +194,245 @@ void PlotCurve::restoreCurveLayout(const QStringList& lst)
 	}
 }
 
+void PlotCurve::drawCurve(QPainter *p, int style, const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const
+{
+	if(d_side_lines)
+		drawSideLines(p, xMap, yMap, from, to);
+	QwtPlotCurve::drawCurve(p, style, xMap, yMap, from, to);
+}
+
+/*!
+  Draw sticks
+
+  \param painter Painter
+  \param xMap x map
+  \param yMap y map
+  \param from index of the first point to be painted
+  \param to index of the last point to be painted
+
+  \sa draw(), drawCurve(), drawDots()
+*/
+void PlotCurve::drawSticks(QPainter *painter,
+    const QwtScaleMap &xMap, const QwtScaleMap &yMap,
+    int from, int to) const
+{
+	if (d_skip_symbols < 2){
+		QwtPlotCurve::drawSticks(painter, xMap, yMap, from, to);
+		return;
+	}
+
+    int x0 = xMap.transform(baseline());
+    int y0 = yMap.transform(baseline());
+
+    for (int i = from; i <= to; i += d_skip_symbols){
+        const int xi = xMap.transform(x(i));
+        const int yi = yMap.transform(y(i));
+
+        if (curveType() == Xfy)
+            QwtPainter::drawLine(painter, x0, yi, xi, yi);
+        else
+            QwtPainter::drawLine(painter, xi, y0, xi, yi);
+    }
+}
+
+void PlotCurve::setSkipSymbolsCount(int count)
+{
+	if (count < 1 || count > dataSize())
+		return;
+
+	d_skip_symbols = count;
+}
+
+/*!
+  \brief Draw symbols
+  \param painter Painter
+  \param symbol Curve symbol
+  \param xMap x map
+  \param yMap y map
+  \param from index of the first point to be painted
+  \param to index of the last point to be painted
+
+  \sa setSymbol(), draw(), drawCurve()
+*/
+void PlotCurve::drawSymbols(QPainter *painter, const QwtSymbol &symbol,
+    const QwtScaleMap &xMap, const QwtScaleMap &yMap,
+    int from, int to) const
+{
+	if (d_skip_symbols < 2){
+		QwtPlotCurve::drawSymbols(painter, symbol, xMap, yMap, from, to);
+		return;
+	}
+
+    painter->setBrush(symbol.brush());
+    painter->setPen(QwtPainter::scaledPen(symbol.pen()));
+
+    const QwtMetricsMap &metricsMap = QwtPainter::metricsMap();
+
+    QRect rect;
+    rect.setSize(metricsMap.screenToLayout(symbol.size()));
+
+	for (int i = from; i <= to; i += d_skip_symbols){
+		const int xi = xMap.transform(x(i));
+		const int yi = yMap.transform(y(i));
+
+		rect.moveCenter(QPoint(xi, yi));
+		symbol.draw(painter, rect);
+	}
+}
+
+void PlotCurve::drawSideLines(QPainter *p, const QwtScaleMap &xMap, const QwtScaleMap &yMap, int from, int to) const
+{
+	if (!p || dataSize() <= 0)
+		return;
+
+	if (to < 0)
+		to = dataSize() - 1;
+
+	p->save();
+	QPen pen = p->pen();
+	pen.setCapStyle(Qt::FlatCap);
+	pen.setJoinStyle(Qt::MiterJoin);
+	p->setPen(pen);
+
+	double lw = 0.5*pen.widthF();
+	const double xl = xMap.xTransform(x(from));
+	const double xr = xMap.xTransform(x(to));
+	const double yl = yMap.xTransform(y(from)) - lw;
+	const double yr = yMap.xTransform(y(to)) - lw;
+	const double base = yMap.xTransform(baseline());
+
+	p->drawLine(QPointF(xl, yl), QPointF(xl, base));
+	p->drawLine(QPointF(xr, yr), QPointF(xr, base));
+
+	p->restore();
+
+	/*int size = to - from + 1;
+    if ( size <= 0 )
+        return;
+
+	size += 2;
+
+	p->save();
+	QPen pen = p->pen();
+	pen.setCapStyle(Qt::FlatCap);
+	pen.setJoinStyle(Qt::MiterJoin);
+	p->setPen(pen);
+
+	const double xl = xMap.xTransform(minXValue());
+	const double xr = xMap.xTransform(maxXValue());
+	const double base = yMap.xTransform(baseline());
+
+    QwtPolygonF polyline;
+    if ( testCurveAttribute(QwtPlotCurve::Fitted) && curveFitter () )
+    {
+        // Transform x and y values to window coordinates
+        // to avoid a distinction between linear and
+        // logarithmic scales.
+
+        QPolygonF points(size);
+        for (int i = from; i <= to; i++)
+        {
+            QwtDoublePoint &p = points[i];
+            p.setX( xMap.xTransform(x(i)) );
+            p.setY( yMap.xTransform(y(i)) );
+        }
+
+        points = curveFitter()->fitCurve(points);
+        size = points.size();
+
+        if ( size == 0 )
+            return;
+
+        // Round QwtDoublePoints to QPoints
+        // When Qwt support for Qt3 has been dropped (Qwt 6.x)
+        // we will use a doubles for painting and the following
+        // step will be obsolete.
+
+        polyline.resize(size);
+
+        const QwtDoublePoint *p = points.data();
+        QPointF *pl = polyline.data();
+        if (testPaintAttribute(QwtPlotCurve::PaintFiltered))
+        {
+
+            QPointF pp(p[0].x(), p[0].y());
+            pl[0] = pp;
+
+            int count = 1;
+            for (int i = 1; i < size; i++)
+            {
+                const QPointF pi(p[i].x(), p[i].y());
+                if ( pi != pp )
+                {
+                    pl[count++] = pi;
+                    pp = pi;
+                }
+            }
+            if ( count != size )
+                polyline.resize(count);
+        }
+        else
+        {
+            for ( int i = 0; i < size; i++ )
+            {
+                pl[i].setX( p[i].x() );
+                pl[i].setY( p[i].y() );
+            }
+        }
+    }
+    else
+    {
+        polyline.resize(size);
+
+        if ( testPaintAttribute(QwtPlotCurve::PaintFiltered) )
+        {
+        	QPointF pp(xl, base);
+            polyline[0] = pp;
+
+            int count = 1;
+            for (int i = from; i <= to; i++)
+            {
+                const QPointF pi(xMap.xTransform(x(i)), yMap.xTransform(y(i)));
+                if ( pi != pp )
+                {
+                    polyline[count] = pi;
+                    count++;
+
+                    pp = pi;
+                }
+            }
+
+            polyline[size - 1] = QPointF(xr, base);
+			count++;
+
+            if ( count != size )
+                polyline.resize(count);
+        }
+        else
+        {
+            polyline[0] = QPointF( xl, base );
+
+            for (int i = from; i <= to; i++)
+            {
+                const QPointF pi(xMap.xTransform(x(i)), yMap.xTransform(y(i)));
+                polyline[i - from + 1] = pi;
+            }
+
+            polyline[size - 1] = QPointF(xr, base);
+        }
+    }
+
+    if ( testPaintAttribute(QwtPlotCurve::ClipPolygons) )
+        polyline = QwtClipper::clipPolygonF(p->window(), polyline);
+
+    QwtPainter::drawPolyline(p, polyline);
+
+    if ( brush().style() != Qt::NoBrush )
+        fillCurve(p, xMap, yMap, polyline);
+
+	p->restore();*/
+}
+
 DataCurve::DataCurve(Table *t, const QString& xColName, const QString& name, int startRow, int endRow):
     PlotCurve(name),
 	d_table(t),
@@ -272,6 +531,21 @@ bool DataCurve::updateData(Table *t, const QString& colName)
 	return true;
 }
 
+void DataCurve::enableSpeedMode()
+{
+	Graph *g = (Graph *)plot();
+	if (!g)
+		return;
+
+	if (g->getDouglasPeukerTolerance() != 0.0 && dataSize() >= g->speedModeMaxPoints()){
+		setCurveAttribute(QwtPlotCurve::Fitted);
+
+		QwtWeedingCurveFitter *fitter = new QwtWeedingCurveFitter(g->getDouglasPeukerTolerance());
+		setCurveFitter(fitter);
+	}
+	//g->replot();
+}
+
 void DataCurve::loadData()
 {
 	Graph *g = (Graph *)plot();
@@ -386,6 +660,8 @@ void DataCurve::loadData()
             g->setLabelsTextFormat(QwtPlot::yLeft, ScaleDraw::Text, title().text(), yLabels);
 	}
 
+	enableSpeedMode();
+
     if (!d_labels_list.isEmpty()){
         ((Graph*)plot())->updatePlot();
         loadLabels();
@@ -425,6 +701,8 @@ void DataCurve::setVisible(bool on)
 	QwtPlotCurve::setVisible(on);
 	foreach(DataCurve *c, d_error_bars)
 		c->setVisible(on);
+	foreach(PlotMarker *m, d_labels_list)
+		m->setVisible(on);
 }
 
 int DataCurve::tableRow(int point)
@@ -477,7 +755,7 @@ int DataCurve::tableRow(int point)
 		if (d_table->cell(i, xcol) == x_val && d_table->cell(i, ycol) == y_val)
 			return i;
 	}
-	return -1;
+	return point;
 }
 
 void DataCurve::setLabelsColumnName(const QString& name)
@@ -706,6 +984,7 @@ void DataCurve::clone(DataCurve* c)
 	d_labels_align = c->labelsAlignment();
 	d_labels_x_offset = c->labelsXOffset();
 	d_labels_y_offset = c->labelsYOffset();
+	d_skip_symbols = c->skipSymbolsCount();
 
     if (!c->labelsColumnName().isEmpty()){
         plot()->replot();
@@ -718,18 +997,18 @@ QString DataCurve::saveToString()
 	if (!validCurveType())
 		return QString();
 
-    if (d_labels_list.isEmpty() || type() == Graph::Function || type() == Graph::Box)
-        return QString();
+	QString s = QString::null;
+	if (d_skip_symbols > 1)
+		s += "<SkipPoints>" + QString::number(d_skip_symbols) + "</SkipPoints>\n";
 
-    QString s = "<CurveLabels>\n";
+    if (d_labels_list.isEmpty() || type() == Graph::Function || type() == Graph::Box)
+        return s;
+
+    s += "<CurveLabels>\n";
     s += "\t<column>" + d_labels_column + "</column>\n";
     s += "\t<color>" + d_labels_color.name() + "</color>\n";
     s += "\t<whiteOut>" + QString::number(d_white_out_labels) + "</whiteOut>\n";
-    s += "\t<font>" + d_labels_font.family() + "\t";
-    s += QString::number(d_labels_font.pointSize()) + "\t";
-    s += QString::number(d_labels_font.bold()) + "\t";
-    s += QString::number(d_labels_font.italic()) + "\t";
-    s += QString::number(d_labels_font.underline()) + "</font>\n";
+    s += "\t<Font>" + d_labels_font.toString() + "</Font>\n";
     s += "\t<angle>" + QString::number(d_labels_angle) + "</angle>\n";
     s += "\t<justify>" + QString::number(d_labels_align) + "</justify>\n";
     if (d_labels_x_offset != 0.0)
@@ -739,22 +1018,68 @@ QString DataCurve::saveToString()
     return s + "</CurveLabels>\n";
 }
 
+void DataCurve::restoreLabels(const QStringList& lst)
+{
+    QString labelsColumn = QString();
+    int xoffset = 0, yoffset = 0;
+  	QStringList::const_iterator line = lst.begin();
+  	QString s = *line;
+    if (s.contains("<column>"))
+        labelsColumn = s.remove("<column>").remove("</column>").trimmed();
+
+  	for (line++; line != lst.end(); line++){
+        s = *line;
+        if (s.contains("<color>"))
+            setLabelsColor(QColor(s.remove("<color>").remove("</color>").trimmed()));
+        else if (s.contains("<whiteOut>"))
+            setLabelsWhiteOut(s.remove("<whiteOut>").remove("</whiteOut>").toInt());
+        else if (s.contains("<font>")){//versions older than 0.9.7.8
+            QStringList fontList = s.remove("<font>").remove("</font>").trimmed().split("\t");
+            QFont font = QFont(fontList[0], fontList[1].toInt());
+            if (fontList.count() >= 3)
+                font.setBold(fontList[2].toInt());
+            if (fontList.count() >= 4)
+                font.setItalic(fontList[3].toInt());
+            if (fontList.count() >= 5)
+                font.setUnderline(fontList[4].toInt());
+            setLabelsFont(font);
+        } else if (s.contains("<Font>")){
+            s.remove("<Font>").remove("</Font>").trimmed();
+            QFont fnt;
+			fnt.fromString(s);
+            setLabelsFont(fnt);
+        }else if (s.contains("<angle>"))
+            setLabelsRotation(s.remove("<angle>").remove("</angle>").toDouble());
+        else if (s.contains("<justify>"))
+            setLabelsAlignment(s.remove("<justify>").remove("</justify>").toInt());
+        else if (s.contains("<xoffset>"))
+            xoffset = s.remove("<xoffset>").remove("</xoffset>").toInt();
+        else if (s.contains("<yoffset>"))
+            yoffset = s.remove("<yoffset>").remove("</yoffset>").toInt();
+    }
+    setLabelsOffset(xoffset, yoffset);
+    setLabelsColumnName(labelsColumn);
+}
+
 bool DataCurve::selectedLabels(const QPoint& pos)
 {
 	if (!validCurveType())
 		return false;
 
     QwtPlot *d_plot = plot();
-    if (!d_plot)
+    if (!d_plot || ((Graph *)d_plot)->hasActiveTool())
         return false;
 
-
     bool selected = false;
 	d_selected_label = NULL;
     foreach(PlotMarker *m, d_labels_list){
         int x = d_plot->transform(xAxis(), m->xValue());
         int y = d_plot->transform(yAxis(), m->yValue());
-        if (QRect(QPoint(x, y), m->label().textSize()).contains(pos)){
+
+        QMatrix wm;
+        wm.translate(x, y);
+		wm.rotate(-d_labels_angle);
+        if (wm.mapToPolygon(QRect(QPoint(0, 0), m->label().textSize())).containsPoint(pos, Qt::OddEvenFill)){
 			d_selected_label = m;
 			d_click_pos_x = d_plot->invTransform(xAxis(), pos.x());
 			d_click_pos_y = d_plot->invTransform(yAxis(), pos.y());
@@ -811,7 +1136,7 @@ bool DataCurve::validCurveType()
 {
 	int style = type();
 	if (style == Graph::Function || style == Graph::Box ||
-		style == Graph::Pie || style == Graph::ErrorBars ||
+		style == Graph::Pie || //style == Graph::ErrorBars ||
 		style == Graph::ColorMap ||  style == Graph::GrayScale ||
 		style == Graph::Contour || style == Graph::ImagePlot)
 		return false;
@@ -848,7 +1173,9 @@ void DataCurve::moveLabels(const QPoint& pos)
 
 PlotMarker::PlotMarker(int index, double angle):QwtPlotMarker(),
 	d_index(index),
-	d_angle(angle)
+	d_angle(angle),
+	d_label_x_offset(0.0),
+	d_label_y_offset(0.0)
 {}
 
 void PlotMarker::draw(QPainter *p, const QwtScaleMap &xMap, const QwtScaleMap &yMap, const QRect &) const
