@@ -31,6 +31,7 @@
 #include <QtGui>
 #include <QHttp>
 #include <QIODevice>
+#include <QNetworkProxy>
 
 #include "EnrichmentDialog.h"
 #include <Graph.h>
@@ -47,6 +48,7 @@
 #include <DoubleSpinBox.h>
 #include <PatternBox.h>
 #include <PenStyleBox.h>
+#include <pixmaps.h>
 
 static const char* choose_folder_xpm[]={
     "16 16 11 1",
@@ -133,6 +135,13 @@ void EnrichmentDialog::initEditorPage()
 {
 	http = new QHttp(this);
     connect(http, SIGNAL(done(bool)), this, SLOT(updateForm(bool)));
+    http->setHost("mathtran.org");
+	QNetworkProxy proxy = QNetworkProxy::applicationProxy();
+	if (!proxy.hostName().isEmpty())
+		http->setProxy(proxy.hostName(), proxy.port(), proxy.user(), proxy.password());
+
+	compileProcess = NULL;
+	dvipngProcess = NULL;
 
 	editPage = new QWidget();
 
@@ -140,16 +149,27 @@ void EnrichmentDialog::initEditorPage()
 
 	texFormatButtons = new TextFormatButtons(equationEditor, TextFormatButtons::Equation);
 
+	texCompilerBox = new QComboBox;
+	texCompilerBox->addItem(tr("MathTran (http://www.mathtran.org/)"));
+	texCompilerBox->addItem(tr("locally installed"));
+	texCompilerBox->setCurrentIndex(((ApplicationWindow *)parentWidget())->d_latex_compiler);
+	connect(texCompilerBox, SIGNAL(activated(int)), this, SLOT(updateCompilerInterface(int)));
+
+	QHBoxLayout *hl = new QHBoxLayout;
+	hl->addWidget(new QLabel(tr("LaTeX Compiler")));
+	hl->addWidget(texCompilerBox);
+
 	outputLabel = new QLabel;
     outputLabel->setFrameShape(QFrame::StyledPanel);
 
 	QVBoxLayout *layout = new QVBoxLayout(editPage);
     layout->addWidget(equationEditor, 1);
 	layout->addWidget(texFormatButtons);
+	layout->addLayout(hl);
 	layout->addWidget(new QLabel(tr("Preview:")));
 	layout->addWidget(outputLabel);
 
-	tabWidget->addTab(editPage, tr( "&Text" ) );
+	tabWidget->addTab(editPage, tr("&Text" ));
 }
 
 void EnrichmentDialog::initTextPage()
@@ -194,6 +214,9 @@ void EnrichmentDialog::initTextPage()
     autoUpdateTextBox = new QCheckBox(tr("Auto-&update"));
 	gl1->addWidget(autoUpdateTextBox, 1, 2);
 
+	texOutputBox = new QCheckBox(tr("TeX &Output"));
+	gl1->addWidget(texOutputBox, 2, 2);
+
 	gl1->setColumnStretch(4, 1);
 
     QVBoxLayout *vl = new QVBoxLayout();
@@ -219,7 +242,6 @@ void EnrichmentDialog::initTextPage()
 
 	textEditBox = new QTextEdit();
 	textEditBox->setTextFormat(Qt::PlainText);
-	//textEditBox->setFont(QFont());
 
 	formatButtons =  new TextFormatButtons(textEditBox, TextFormatButtons::Legend);
 
@@ -403,6 +425,16 @@ void EnrichmentDialog::initGeometryPage()
 {
     geometryPage = new QWidget();
 
+	QGridLayout *bl1 = new QGridLayout();
+
+	attachToBox = new QComboBox();
+	attachToBox->insertItem(tr("Page"));
+	attachToBox->insertItem(tr("Layer Scales"));
+	//attachToBox->setCurrentIndex((int)lm->attachPolicy());
+
+	bl1->addWidget(new QLabel(tr( "Attach to" )), 0, 0);
+	bl1->addWidget(attachToBox, 0, 1);
+
 	unitBox = new QComboBox();
 	unitBox->insertItem(tr("inch"));
 	unitBox->insertItem(tr("mm"));
@@ -412,9 +444,8 @@ void EnrichmentDialog::initGeometryPage()
 	if (d_widget_type != MDIWindow)
         unitBox->insertItem(tr("scale"));
 
-	QBoxLayout *bl1 = new QBoxLayout (QBoxLayout::LeftToRight);
-	bl1->addWidget(new QLabel(tr( "Unit" )));
-	bl1->addWidget(unitBox);
+	bl1->addWidget(new QLabel(tr( "Unit" )), 1, 0);
+	bl1->addWidget(unitBox, 1, 1);
 
 	ApplicationWindow *app = (ApplicationWindow *)parent();
 	QLocale locale = QLocale();
@@ -529,6 +560,7 @@ void EnrichmentDialog::setWidget(QWidget *w)
 		boxFrameWidth->blockSignals(false);
 
 		unitBox->setCurrentIndex(app->d_frame_geometry_unit);
+		attachToBox->setCurrentIndex((int)fw->attachPolicy());
 		displayCoordinates(app->d_frame_geometry_unit);
     } else {
 		unitBox->setCurrentIndex(FrameWidget::Pixel);
@@ -561,6 +593,7 @@ void EnrichmentDialog::setWidget(QWidget *w)
             boxTextAngle->setValue(l->angle());
 			boxTextAngle->blockSignals(false);
 			autoUpdateTextBox->setChecked(l->isAutoUpdateEnabled());
+			texOutputBox->setChecked(l->hasTeXOutput());
 		}
 	} else if (d_widget_type == Tex){
 		TexWidget *tw = qobject_cast<TexWidget *>(d_widget);
@@ -633,9 +666,12 @@ void EnrichmentDialog::apply()
 		frameApplyTo();
 	else if (imagePage && tabWidget->currentPage() == imagePage)
 		chooseImageFile(imagePathBox->text());
-	else if (tabWidget->currentPage() == geometryPage)
+	else if (tabWidget->currentPage() == geometryPage){
 		setCoordinates(unitBox->currentIndex());
-	else if (patternPage && tabWidget->currentPage() == patternPage)
+		FrameWidget *fw = qobject_cast<FrameWidget *>(d_widget);
+        if (fw)
+            fw->setAttachPolicy((FrameWidget::AttachPolicy)attachToBox->currentIndex());
+	} else if (patternPage && tabWidget->currentPage() == patternPage)
 		patternApplyTo();
 	else if (textPage && tabWidget->currentPage() == textPage){
 		LegendWidget *l = qobject_cast<LegendWidget *>(d_widget);
@@ -649,6 +685,30 @@ void EnrichmentDialog::apply()
 	}
 }
 
+QString EnrichmentDialog::createTempTexFile()
+{
+	QString path = QDir::tempPath();
+	QString name = path + "/" + "QtiPlot_temp.tex";
+
+	QFile file(name);
+
+	if (file.open(QIODevice::WriteOnly)){
+		QTextStream t( &file );
+		t.setEncoding(QTextStream::UnicodeUTF8);
+		t << "\\documentclass{article}\n";
+		t << "\\pagestyle{empty}\n";
+		t << "\\begin{document}\n";
+		t << "\\huge{\\mbox{$";
+		t << equationEditor->toPlainText();
+		t << "$}}\n";
+		t << "\\end{document}";
+		file.close();
+		return QDir::cleanPath(name);
+	}
+
+	return QString();
+}
+
 void EnrichmentDialog::fetchImage()
 {
 	TexWidget *tw = qobject_cast<TexWidget *>(d_widget);
@@ -659,6 +719,28 @@ void EnrichmentDialog::fetchImage()
     updateButton->setEnabled(false);
     equationEditor->setReadOnly(true);
 
+	if (texCompilerBox->currentIndex() == 1){
+		if (compileProcess)
+			delete compileProcess;
+
+		compileProcess = new QProcess(this);
+		connect(compileProcess, SIGNAL(finished(int, QProcess::ExitStatus)),
+			this, SLOT(finishedCompiling(int, QProcess::ExitStatus)));
+		connect(compileProcess, SIGNAL(error(QProcess::ProcessError)),
+			this, SLOT(displayCompileError(QProcess::ProcessError)));
+
+		compileProcess->setWorkingDirectory(QDir::tempPath());
+
+		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+
+		QString program = ((ApplicationWindow *)parentWidget())->d_latex_compiler_path;
+		QStringList arguments;
+		arguments << createTempTexFile();
+
+		compileProcess->start(program, arguments);
+		return;
+	}
+
     QUrl url;
     url.setPath("/cgi-bin/mathtran");
     url.setQueryDelimiters('=', ';');
@@ -666,7 +748,6 @@ void EnrichmentDialog::fetchImage()
     url.addQueryItem("tex", QUrl::toPercentEncoding(
                      equationEditor->toPlainText()));
 
-    http->setHost("mathtran.org");
     http->get(url.toString());
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
@@ -707,7 +788,8 @@ void EnrichmentDialog::chooseImageFile(const QString& fn)
 
 	QString path = fn;
 	if (path.isEmpty())
-		path = QFileDialog::getOpenFileName(this, tr("QtiPlot - Import image from file"), app->imagesDirPath);
+		path = ApplicationWindow::getFileName(this, tr("QtiPlot - Import image from file"), app->imagesDirPath,
+					ApplicationWindow::imageFilter(), 0, false);
 
 	if (!path.isEmpty()){
 		ImageWidget *i = qobject_cast<ImageWidget *>(d_widget);
@@ -1071,6 +1153,7 @@ void EnrichmentDialog::setTextFormatTo(LegendWidget *l)
     l->setFont(textFont);
     l->setAngle(boxTextAngle->value());
     l->setAutoUpdate(autoUpdateTextBox->isChecked());
+    l->setTeXOutput(texOutputBox->isChecked());
     l->repaint();
 }
 
@@ -1119,6 +1202,137 @@ void EnrichmentDialog::setRectangleDefaultValues()
 	app->saveSettings();
 }
 
+void EnrichmentDialog::createImage()
+{
+	QApplication::restoreOverrideCursor();
+
+	QString path = QDir::tempPath();
+	QString fileName = QDir::cleanPath(path + "/" + "QtiPlot_temp.png");
+
+	QImage image;
+	if (image.load(fileName)){
+		QPixmap pixmap = QPixmap::fromImage(image);
+		outputLabel->setPixmap(pixmap);
+		TexWidget *tw = qobject_cast<TexWidget *>(d_widget);
+		if (tw){
+			tw->setPixmap(pixmap);
+			tw->setFormula(equationEditor->toPlainText());
+			d_plot->multiLayer()->notifyChanges();
+		}
+	}
+
+    clearButton->setEnabled(true);
+    updateButton->setEnabled(true);
+    equationEditor->setReadOnly(false);
+
+	if (dvipngProcess)
+		delete dvipngProcess;
+    dvipngProcess = NULL;
+    QFile::remove(fileName);
+
+    fileName = QDir::cleanPath(path + "/" + "QtiPlot_temp.dvi");
+    QFile::remove(fileName);
+}
+
+void EnrichmentDialog::finishedCompiling(int exitCode, QProcess::ExitStatus exitStatus)
+{
+	QApplication::restoreOverrideCursor();
+	if (exitStatus != QProcess::NormalExit){
+		QMessageBox::critical(this, tr("Compile process ended"),
+		tr("Compiling process ended with exit code: %1").arg(exitCode));
+		return;
+	}
+
+	QString compiler = ((ApplicationWindow *)parentWidget())->d_latex_compiler_path;
+	QFileInfo fi(compiler);
+
+	QString dir = fi.dir().absolutePath();
+        QString program = dir + "/dvipng";
+        #ifdef Q_OS_WIN
+            program += ".exe";
+        #endif
+
+	QStringList arguments;
+	arguments << "-T" << "tight" << "QtiPlot_temp.dvi" << "-o" << "QtiPlot_temp.png";
+
+	if (compileProcess)
+		delete compileProcess;
+	compileProcess = NULL;
+
+	dvipngProcess = new QProcess(this);
+	connect(dvipngProcess, SIGNAL(error(QProcess::ProcessError)),
+		this, SLOT(displayCompileError(QProcess::ProcessError)));
+	connect(dvipngProcess, SIGNAL(finished(int, QProcess::ExitStatus)),
+		this, SLOT(createImage()));
+
+	dvipngProcess->setWorkingDirectory (QDir::tempPath());
+
+	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+	dvipngProcess->start(program, arguments);
+	return;
+}
+
+void EnrichmentDialog::displayCompileError(QProcess::ProcessError error)
+{
+	QString process = QString::null;
+	if (compileProcess)
+		process = tr("LaTeX compile process");
+	else if (dvipngProcess)
+		process = tr("dvipng process");
+
+	QString msg;
+	switch(error){
+		case QProcess::FailedToStart:
+			if (compileProcess)
+				msg = process + " " + tr("failed to start!") + " " +
+				tr("Please set the correct path to the compiler in the preferences dialog!");
+			else if (dvipngProcess)
+				msg = process + " " + tr("failed to start!") + " " +
+				tr("Please verify that you have dvipng installed in the same folder as your LaTeX compiler!");
+		break;
+
+		case QProcess::Crashed:
+			msg = process + " " + tr("crashed");
+		break;
+
+		case QProcess::Timedout:
+			msg = process + " " + tr("timedout");
+		break;
+
+		case QProcess::WriteError:
+			msg = process + " " + tr("write error");
+		break;
+
+		case QProcess::ReadError:
+			msg = process + " " + tr("read error");
+		break;
+
+		case QProcess::UnknownError:
+			msg =  process + " " + tr("unknown error");
+		break;
+	}
+
+	QApplication::restoreOverrideCursor();
+	if (compileProcess){
+		compileProcess->kill();
+		compileProcess = NULL;
+	} else if (dvipngProcess){
+		dvipngProcess->kill();
+		dvipngProcess = NULL;
+	}
+
+	QMessageBox::critical(this, tr("Compile error"), msg);
+
+	clearButton->setEnabled(true);
+    updateButton->setEnabled(true);
+    equationEditor->setReadOnly(false);
+}
+
+void EnrichmentDialog::updateCompilerInterface(int compiler)
+{
+	((ApplicationWindow *)parentWidget())->d_latex_compiler = compiler;
+}
+
 EnrichmentDialog::~EnrichmentDialog()
 {
 	QApplication::restoreOverrideCursor();
