@@ -44,12 +44,14 @@
 #include <QCursor>
 #include <QImageWriter>
 #include <QTextStream>
+#include <QTextDocumentWriter>
 
 #include <qwt3d_io_gl2ps.h>
 #include <qwt3d_coordsys.h>
 #include <qwt_color_map.h>
 
 #include <gsl/gsl_vector.h>
+#include <gsl/gsl_math.h>
 #include <fstream>
 
 UserFunction::UserFunction(const QString& s, SurfacePlot& pw)
@@ -148,6 +150,8 @@ Graph3D::Graph3D(const QString& label, ApplicationWindow* parent, const char* na
 
 void Graph3D::initPlot()
 {
+	setAcceptDrops(true);
+
 	d_table = 0;
 	d_table_plot_type = NoTable;
 	d_matrix = 0;
@@ -158,7 +162,7 @@ void Graph3D::initPlot()
 	connect(d_timer, SIGNAL(timeout()), this, SLOT(rotate()));
 	ignoreFonts = false;
 
-	setGeometry(0, 0, 500, 400);
+	resize(500, 400);
 
 	ApplicationWindow *app = applicationWindow();
 
@@ -172,6 +176,7 @@ void Graph3D::initPlot()
 	sp->setSmoothMesh(app->d_3D_smooth_mesh);
 	sp->setResolution(app->d_3D_resolution);
 	sp->setFloorStyle((Qwt3D::FLOORSTYLE)app->d_3D_projection);
+	sp->setLocale(app->locale());
 	setWidget(sp);
 
 	d_autoscale = app->d_3D_autoscale;
@@ -227,7 +232,13 @@ void Graph3D::initPlot()
 	setNumbersColor(app->d_3D_numbers_color);
 	setLabelsColor(app->d_3D_labels_color);
 	setBackgroundColor(app->d_3D_background_color);
-	setGridColor(app->d_3D_grid_color);
+
+	Qwt3D::GridLine majorGrid(app->d_3D_major_grids, Qt2GL(app->d_3D_grid_color), (Qwt3D::LINESTYLE)app->d_3D_major_style, app->d_3D_major_width);
+	Qwt3D::GridLine minorGrid(app->d_3D_minor_grids, Qt2GL(app->d_3D_minor_grid_color), (Qwt3D::LINESTYLE)app->d_3D_minor_style, app->d_3D_minor_width);
+	for (int i = 0; i < 12; i++){
+		sp->coordinates()->setMajorGridLines((Qwt3D::AXIS)i, majorGrid);
+		sp->coordinates()->setMinorGridLines((Qwt3D::AXIS)i, minorGrid);
+	}
 
 	connect(sp,SIGNAL(rotationChanged(double, double, double)),this,SLOT(rotationChanged(double, double, double)));
 	connect(sp,SIGNAL(zoomChanged(double)),this,SLOT(zoomChanged(double)));
@@ -271,10 +282,13 @@ void Graph3D::initCoord()
 void Graph3D::addFunction(const QString& s, double xl, double xr, double yl,
 						double yr, double zl, double zr, int columns, int rows)
 {
-	if (d_surface)
+	if (d_surface){
 		delete d_surface;
-	else if (d_func)
+		d_surface = 0;
+	} else if (d_func){
 		delete d_func;
+		d_func = 0;
+	}
 
 	sp->makeCurrent();
 
@@ -287,6 +301,7 @@ void Graph3D::addFunction(const QString& s, double xl, double xr, double yl,
 	d_func->create();
 
 	sp->legend()->setLimits(zl, zr);
+	sp->legend()->setMajors(legendMajorTicks);
 
 	if (sp->plotStyle() == NOPLOT){
 		sp->setPlotStyle(FILLED);
@@ -302,13 +317,15 @@ void Graph3D::addParametricSurface(const QString& xFormula, const QString& yForm
 						const QString& zFormula, double ul, double ur, double vl, double vr,
 						int columns, int rows, bool uPeriodic, bool vPeriodic)
 {
-	if (d_surface)
+	if (d_surface){
 		delete d_surface;
-	else if (d_func)
+		d_surface = 0;
+	} else if (d_func){
 		delete d_func;
+		d_func = 0;
+	}
 
 	sp->makeCurrent();
-	sp->resize(this->size());
 
 	d_surface = new UserParametricSurface(xFormula, yFormula, zFormula, *sp);
 	d_surface->setMesh(columns, rows);
@@ -319,13 +336,13 @@ void Graph3D::addParametricSurface(const QString& xFormula, const QString& yForm
 	double zl, zr;
 	sp->coordinates()->axes[Z1].limits(zl, zr);
 	sp->legend()->setLimits(zl, zr);
+	sp->legend()->setMajors(legendMajorTicks);
 
 	if (sp->plotStyle() == NOPLOT){
 		sp->setPlotStyle(FILLED);
 		style_=FILLED;
 		pointStyle = None;
 	}
-  	//sp->createCoordinateSystem(Triple(xl, yl, zl), Triple(xr, yr, zr));
 	findBestLayout();
 	update();
 }
@@ -447,6 +464,8 @@ void Graph3D::addMatrixData(Matrix* m)
 	if (!m || d_matrix == m)
 		return;
 
+	d_table = NULL;
+
 	bool first_time = false;
 	if(!d_matrix)
 		first_time = true;
@@ -481,7 +500,11 @@ void Graph3D::addMatrixData(Matrix* m)
 void Graph3D::addMatrixData(Matrix* m, double xl, double xr,
 		double yl, double yr, double zl, double zr)
 {
+	if (!m)
+		return;
+
 	d_matrix = m;
+	d_table = NULL;
 	plotAssociation = "matrix<" + QString(m->objectName()) + ">";
 
 	updateScalesFromMatrix(xl, xr, yl, yr, zl, zr);
@@ -556,6 +579,7 @@ void Graph3D::loadData(Table* table, int xCol, int yCol, int zCol,
 		return;
 
 	d_table = table;
+	d_matrix = NULL;
 
 	plotAssociation = table->colName(xCol) + "(X),";
 	plotAssociation += table->colName(yCol) + "(Y),";
@@ -995,6 +1019,9 @@ void Graph3D::resetAxesLabels()
 
 void Graph3D::setAxesLabels(const QStringList& l)
 {
+	if (l.isEmpty())
+		return;
+
 	QString label = l[0];
 	sp->coordinates()->axes[X1].setLabelString(label);
 	sp->coordinates()->axes[X2].setLabelString(label);
@@ -1118,45 +1145,6 @@ double Graph3D::zStop()
 	return stop;
 }
 
-QStringList Graph3D::scaleLimits()
-{
-	QStringList limits;
-	double start, stop;
-	int majors, minors;
-
-	sp->coordinates()->axes[X1].limits (start,stop);
-	majors=sp->coordinates()->axes[X1].majors();
-	minors=sp->coordinates()->axes[X1].minors();
-
-	limits<<QString::number(start);
-	limits<<QString::number(stop);
-	limits<<QString::number(majors);
-	limits<<QString::number(minors);
-	limits<<QString::number(scaleType[0]);
-
-	sp->coordinates()->axes[Y1].limits (start,stop);
-	majors=sp->coordinates()->axes[Y1].majors();
-	minors=sp->coordinates()->axes[Y1].minors();
-
-	limits<<QString::number(start);
-	limits<<QString::number(stop);
-	limits<<QString::number(majors);
-	limits<<QString::number(minors);
-	limits<<QString::number(scaleType[1]);
-
-	sp->coordinates()->axes[Z1].limits (start,stop);
-	majors=sp->coordinates()->axes[Z1].majors();
-	minors=sp->coordinates()->axes[Z1].minors();
-
-	limits<<QString::number(start);
-	limits<<QString::number(stop);
-	limits<<QString::number(majors);
-	limits<<QString::number(minors);
-	limits<<QString::number(scaleType[2]);
-
-	return limits;
-}
-
 QStringList Graph3D::scaleTicks()
 {
 	QStringList limits;
@@ -1182,14 +1170,25 @@ QStringList Graph3D::scaleTicks()
 
 void Graph3D::setScale(int axis, double start, double end, int majorTicks, int minorTicks, Qwt3D::SCALETYPE type)
 {
+	double left, right;
+	sp->coordinates()->axes[axis].limits(left, right);
+
+	if (left == start && right == end &&
+		sp->coordinates()->axes[axis].majors() == majorTicks &&
+		sp->coordinates()->axes[axis].minors() == minorTicks &&
+		scaleType[axis] == type) return;
+
 	double xMin, xMax, yMin, yMax, zMin, zMax;
-	int baseAxis;
+	int axis1 = X1, axis2 = X2, axis3 = X3, axis4 = X4;
 
 	sp->makeCurrent();
 	sp->coordinates()->axes[X1].limits(xMin, xMax);
 	sp->coordinates()->axes[Y1].limits(yMin, yMax);
 	sp->coordinates()->axes[Z1].limits(zMin, zMax);
 
+	double majorTicLength, minorTicLength;
+	sp->coordinates()->axes[axis].ticLength(majorTicLength, minorTicLength);
+
 	switch(axis)
 	{
 	case 0:
@@ -1198,7 +1197,6 @@ void Graph3D::setScale(int axis, double start, double end, int majorTicks, int m
 			xMin = start;
 			xMax = end;
 		}
-		baseAxis = X1;
 		break;
 	case 1:
 		if (yMin != start || yMax != end)
@@ -1206,7 +1204,7 @@ void Graph3D::setScale(int axis, double start, double end, int majorTicks, int m
 			yMin = start;
 			yMax = end;
 		}
-		baseAxis = Y1;
+		axis1 = Y1, axis2 = Y2, axis3 = Y3, axis4 = Y4;
 		break;
 	case 2:
 		if (zMin != start || zMax != end)
@@ -1214,7 +1212,7 @@ void Graph3D::setScale(int axis, double start, double end, int majorTicks, int m
 			zMin = start;
 			zMax = end;
 		}
-		baseAxis = Z1;
+		axis1 = Z1, axis2 = Z2, axis3 = Z3, axis4 = Z4;
 		break;
 	}
 
@@ -1223,195 +1221,42 @@ void Graph3D::setScale(int axis, double start, double end, int majorTicks, int m
 		d_func->setMinZ(zMin);
 		d_func->setMaxZ(zMax);
 		d_func->create();
-		sp->createCoordinateSystem(Triple(xMin, yMin, zMin), Triple(xMax, yMax, zMax));
 	} else if (d_surface){
 		d_surface->restrictRange(ParallelEpiped(Triple(xMin, yMin, zMin), Triple(xMax, yMax, zMax)));
 		d_surface->create();
-		sp->createCoordinateSystem(Triple(xMin, yMin, zMin), Triple(xMax, yMax, zMax));
 	} else
 		setScales(xMin, xMax, yMin, yMax, zMin, zMax);
 
+	sp->coordinates()->axes[axis1].setLimits(start, end);
+	sp->coordinates()->axes[axis2].setLimits(start, end);
+	sp->coordinates()->axes[axis3].setLimits(start, end);
+	sp->coordinates()->axes[axis4].setLimits(start, end);
+
+	resetAxesType();
 	if(scaleType[axis] != type){
-		sp->coordinates()->axes[baseAxis].setScale(type);
+		sp->coordinates()->axes[axis1].setScale(type);
+		sp->coordinates()->axes[axis2].setScale(type);
+		sp->coordinates()->axes[axis3].setScale(type);
+		sp->coordinates()->axes[axis4].setScale(type);
 		scaleType[axis] = type;
 	}
 
-	if (sp->coordinates()->axes[baseAxis].majors() != majorTicks){
-		sp->coordinates()->axes[baseAxis].setMajors(majorTicks);
-		sp->coordinates()->axes[(baseAxis+1)*3].setMajors(majorTicks);
-		sp->coordinates()->axes[(baseAxis+1)*3+1].setMajors(majorTicks);
-		sp->coordinates()->axes[(baseAxis+1)*3+2].setMajors(majorTicks);
+	if (sp->coordinates()->axes[axis1].majors() != majorTicks){
+		sp->coordinates()->axes[axis1].setMajors(majorTicks);
+		sp->coordinates()->axes[axis2].setMajors(majorTicks);
+		sp->coordinates()->axes[axis3].setMajors(majorTicks);
+		sp->coordinates()->axes[axis4].setMajors(majorTicks);
 	}
 
-	if (sp->coordinates()->axes[baseAxis].minors() != minorTicks){
-		sp->coordinates()->axes[baseAxis].setMinors(minorTicks);
-		sp->coordinates()->axes[(baseAxis+1)*3].setMinors(minorTicks);
-		sp->coordinates()->axes[(baseAxis+1)*3+1].setMinors(minorTicks);
-		sp->coordinates()->axes[(baseAxis+1)*3+2].setMinors(minorTicks);
+	if (sp->coordinates()->axes[axis1].minors() != minorTicks){
+		sp->coordinates()->axes[axis1].setMinors(minorTicks);
+		sp->coordinates()->axes[axis2].setMinors(minorTicks);
+		sp->coordinates()->axes[axis3].setMinors(minorTicks);
+		sp->coordinates()->axes[axis4].setMinors(minorTicks);
 	}
 
-	update();
-	emit modified();
-}
+	sp->coordinates()->axes[axis].setTicLength(majorTicLength, minorTicLength);
 
-void Graph3D::updateScale(int axis, const QStringList& options)
-{
-	QString st = QString::number(scaleType[axis]);
-	double start, stop, xl, xr, yl, yr;
-	int majors, minors, newMaj, newMin;
-
-	sp->makeCurrent();
-	switch(axis)
-	{
-		case 0:
-			majors=sp->coordinates()->axes[X1].majors ();
-  	        minors=sp->coordinates()->axes[X1].minors ();
-			sp->coordinates()->axes[X1].limits(xl,xr);
-			if (xl !=options[0].toDouble() || xr != options[1].toDouble())
-			{
-				xl=options[0].toDouble();
-				xr=options[1].toDouble();
-				sp->coordinates()->axes[Y1].limits(yl,yr);
-				sp->coordinates()->axes[Z1].limits(start,stop);
-
-				if (d_func){
-					d_func->setDomain(xl,xr,yl,yr);
-					d_func->create ();
-					sp->createCoordinateSystem(Triple(xl, yl, start), Triple(xr, yr, stop));
-				} else if (d_surface){
-					d_surface->restrictRange (ParallelEpiped(Triple(xl, yl, start), Triple(xr, yr, stop)));
-					d_surface->create();
-					sp->createCoordinateSystem(Triple(xl, yl, start), Triple(xr, yr, stop));
-				} else
-					setScales(xl, xr, yl, yr, start, stop);
-			}
-
-			if(st != options[4]){
-				if (options[4]=="0"){
-					sp->coordinates()->axes[X1].setScale (LINEARSCALE);
-					scaleType[axis]=0;
-				} else {
-					sp->coordinates()->axes[X1].setScale (LOG10SCALE);
-					scaleType[axis]=1;
-				}
-			}
-
-			newMaj = options[2].toInt();
-			if (majors != newMaj){
-				sp->coordinates()->axes[X1].setMajors(newMaj);
-				sp->coordinates()->axes[X2].setMajors(newMaj);
-				sp->coordinates()->axes[X3].setMajors(newMaj);
-				sp->coordinates()->axes[X4].setMajors(newMaj);
-			}
-
-			newMin = options[3].toInt();
-			if (minors != newMin){
-				sp->coordinates()->axes[X1].setMinors(newMin);
-				sp->coordinates()->axes[X2].setMinors(newMin);
-				sp->coordinates()->axes[X3].setMinors(newMin);
-				sp->coordinates()->axes[X4].setMinors(newMin);
-			}
-			break;
-
-		case 1:
-			majors = sp->coordinates()->axes[Y1].majors ();
-  	        minors = sp->coordinates()->axes[Y1].minors ();
-			sp->coordinates()->axes[Y1].limits(yl, yr);
-			if (yl != options[0].toDouble() || yr != options[1].toDouble()){
-				yl = options[0].toDouble();
-				yr = options[1].toDouble();
-				sp->coordinates()->axes[X1].limits(xl, xr);
-				sp->coordinates()->axes[Z1].limits(start, stop);
-
-				if (d_func){
-					d_func->setDomain(xl, xr, yl, yr);
-					d_func->create();
-					sp->createCoordinateSystem(Triple(xl, yl, start), Triple(xr, yr, stop));
-				} else if (d_surface){
-					d_surface->restrictRange (ParallelEpiped(Triple(xl, yl, start), Triple(xr, yr, stop)));
-					d_surface->create();
-					sp->createCoordinateSystem(Triple(xl, yl, start), Triple(xr, yr, stop));
-				} else
-					setScales(xl, xr, yl, yr, start, stop);
-			}
-
-			newMaj = options[2].toInt();
-			if (majors != newMaj ){
-				sp->coordinates()->axes[Y1].setMajors(newMaj);
-				sp->coordinates()->axes[Y2].setMajors(newMaj);
-				sp->coordinates()->axes[Y3].setMajors(newMaj);
-				sp->coordinates()->axes[Y4].setMajors(newMaj);
-			}
-
-            newMin = options[3].toInt();
-			if (minors != newMin){
-				sp->coordinates()->axes[Y1].setMinors(newMin);
-				sp->coordinates()->axes[Y2].setMinors(newMin);
-				sp->coordinates()->axes[Y3].setMinors(newMin);
-				sp->coordinates()->axes[Y4].setMinors(newMin);
-			}
-
-			if(st != options[4]){
-				if (options[4]=="0"){
-					sp->coordinates()->axes[Y1].setScale (LINEARSCALE);
-					scaleType[axis]=0;
-				} else {
-					sp->coordinates()->axes[Y1].setScale (LOG10SCALE);
-					scaleType[axis]=1;
-				}
-			}
-			break;
-
-		case 2:
-			majors=sp->coordinates()->axes[Z1].majors();
-			minors=sp->coordinates()->axes[Z1].minors();
-			sp->coordinates()->axes[Z1].limits(start,stop);
-			if (start != options[0].toDouble() || stop != options[1].toDouble())
-			{
-				start=options[0].toDouble();
-				stop=options[1].toDouble();
-				sp->coordinates()->axes[X1].limits(xl,xr);
-  	            sp->coordinates()->axes[Y1].limits(yl,yr);
-				if (d_func){
-					d_func->setMinZ(start);
-					d_func->setMaxZ(stop);
-					d_func->create ();
-					sp->createCoordinateSystem(Triple(xl, yl, start), Triple(xr, yr, stop));
-				} else if (d_surface){
-					d_surface->restrictRange (ParallelEpiped(Triple(xl, yl, start), Triple(xr, yr, stop)));
-					d_surface->create();
-					sp->createCoordinateSystem(Triple(xl, yl, start), Triple(xr, yr, stop));
-				} else
-					setScales(xl, xr, yl, yr, start, stop);
-				sp->legend()->setLimits(start,stop);
-			}
-
-			newMaj= options[2].toInt();
-			if (majors != newMaj ){
-				sp->coordinates()->axes[Z1].setMajors(newMaj);
-				sp->coordinates()->axes[Z2].setMajors(newMaj);
-				sp->coordinates()->axes[Z3].setMajors(newMaj);
-				sp->coordinates()->axes[Z4].setMajors(newMaj);
-			}
-
-			newMin = options[3].toInt();
-			if (minors != newMin){
-				sp->coordinates()->axes[Z1].setMinors(newMin);
-				sp->coordinates()->axes[Z2].setMinors(newMin);
-				sp->coordinates()->axes[Z3].setMinors(newMin);
-				sp->coordinates()->axes[Z4].setMinors(newMin);
-			}
-			if(st != options[4]){
-  	         	if (options[4]=="0"){
-  	             	sp->coordinates()->axes[Z1].setScale (LINEARSCALE);
-  	            	scaleType[axis]=0;
-  	            } else {
-  	            	sp->coordinates()->axes[Z1].setScale (LOG10SCALE);
-  	                scaleType[axis]=1;
-  	            }
-  	        }
-			break;
-	}
 	update();
 	emit modified();
 }
@@ -1420,6 +1265,13 @@ void Graph3D::setScales(double xl, double xr, double yl, double yr, double zl, d
 {
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
+	sp->hide();//in order to avoid flickering, due to changes in tick length
+
+	double *majorTicLengths = new double[12];
+	double *minorTicLengths = new double[12];
+	for (int i = 0; i < 12; i++)
+		sp->coordinates()->axes[i].ticLength(majorTicLengths[i], minorTicLengths[i]);
+
 	if (d_matrix)
 		updateScalesFromMatrix(xl, xr, yl, yr, zl, zr);
     else if (d_func){
@@ -1452,6 +1304,16 @@ void Graph3D::setScales(double xl, double xr, double yl, double yr, double zl, d
 			updateScales(xl, xr, yl, yr, zl, zr, xCol, yCol);
 	}
     resetAxesLabels();
+    findBestLayout();
+
+	for (int i = 0; i < 12; i++)
+		sp->coordinates()->axes[i].setTicLength(majorTicLengths[i], minorTicLengths[i]);
+
+	delete [] majorTicLengths;
+	delete [] minorTicLengths;
+
+    sp->show();
+
 	QApplication::restoreOverrideCursor();
 }
 
@@ -1465,37 +1327,47 @@ void Graph3D::updateScalesFromMatrix(double xl, double xr, double yl, double yr,
 	double dy = d_matrix->dy();
     double x_begin = qMin(xl, xr);
 	double y_begin = qMin(yl, yr);
-	int nc = qRound(fabs(xr - xl)/dx) + 1; // new number of columns
-	int nr = qRound(fabs(yr - yl)/dy) + 1; // new number of rows
+	int nc = int(fabs(xr - xl)/dx) + 1;// new number of columns
+	int nr = int(fabs(yr - yl)/dy) + 1;// new number of rows
 	double **data_matrix = Matrix::allocateMatrixData(nc, nr);
 	for (int i = 0; i < nc; i++){
 		double x = x_begin + i*dx;
+		if (x < xStart || x > xEnd){
+			for (int j = 0; j < nr; j++)
+				data_matrix[i][j] = GSL_NAN;
+			continue;
+		}
+
         double dli, dlf;
         dlf = modf(fabs((x - xStart)/dx), &dli);
-        int l = (int)dli; if (dlf > 0.5) l++;
+        int l = int(dli); if (dlf > 0.5) l++;
 		for (int j = 0; j < nr; j++){
 			double y = y_begin + j*dy;
-			if (x >= xStart && x <= xEnd && y >= yStart && y <= yEnd){
-                double dki, dkf;
-                dkf = modf(fabs((y - yStart)/dy), &dki);
-                int k = (int)dki; if (dkf > 0.5) k++;
-				double val = d_matrix->cell(k, l);
-				if (val > zr)
-					data_matrix[i][j] = zr;
-				else if (val < zl)
-					data_matrix[i][j] = zl;
-				else
-					data_matrix[i][j] = val;
-			} else
-				data_matrix[i][j] = 0.0;
+			if (y < yStart || y > yEnd){
+				data_matrix[i][j] = GSL_NAN;
+				continue;
+			}
+
+			double dki, dkf;
+			dkf = modf(fabs((y - yStart)/dy), &dki);
+			int k = int(dki); if (dkf > 0.5) k++;
+			double val = d_matrix->cell(k, l);
+
+			if (val > zr)
+				data_matrix[i][j] = zr;
+			else if (val < zl)
+				data_matrix[i][j] = zl;
+			else
+				data_matrix[i][j] = val;
 		}
 	}
 	sp->loadFromData(data_matrix, nc, nr, xl, xr, yl, yr);
 	Matrix::freeMatrixData(data_matrix, nc);
 
-	sp->createCoordinateSystem(Triple(xl, yl, zl), Triple(xr, yr, zr));
+	sp->coordinates()->setPosition(Triple(xl, yl, zl), Triple(xr, yr, zr));
 	sp->legend()->setLimits(zl, zr);
 	sp->legend()->setMajors(legendMajorTicks);
+
 	update();
 }
 
@@ -1651,7 +1523,7 @@ void Graph3D::setNumbersColor(const QColor& numColor)
 
 void Graph3D::setLabelsColor(const QColor& labelColor)
 {
-	if(labelsCol !=labelColor){
+	if(labelsCol != labelColor){
 		sp->coordinates()->setLabelColor(Qt2GL(labelColor));
 		labelsCol=labelColor;
 	}
@@ -1659,9 +1531,9 @@ void Graph3D::setLabelsColor(const QColor& labelColor)
 
 void Graph3D::setBackgroundColor(const QColor& bgColor)
 {
-	if(bgCol !=bgColor){
+	if(bgCol != bgColor){
 		sp->setBackgroundColor(Qt2GL(bgColor));
-		bgCol=bgColor;
+		bgCol = bgColor;
 	}
 }
 
@@ -1669,7 +1541,15 @@ void Graph3D::setGridColor(const QColor& gridColor)
 {
 	if(gridCol != gridColor){
 		sp->coordinates()->setGridLinesColor(Qt2GL(gridColor));
-		gridCol=gridColor;
+
+		Qwt3D::GridLine majorGrid(true, Qt2GL(gridColor), Qwt3D::SOLID, 1.0);
+		Qwt3D::GridLine minorGrid(true, Qt2GL(gridColor), Qwt3D::DASH, 0.5);
+		for (int i = 0; i < 12; i++){
+			sp->coordinates()->setMajorGridLines((Qwt3D::AXIS)i, majorGrid);
+			sp->coordinates()->setMinorGridLines((Qwt3D::AXIS)i, minorGrid);
+		}
+
+		gridCol = gridColor;
 	}
 }
 
@@ -1929,7 +1809,7 @@ void Graph3D::setGrid(Qwt3D::SIDE s, bool b)
 	else
 		sum &= ~s;
 
-	sp->coordinates()->setGridLines(sum!=Qwt3D::NOSIDEGRID, false, sum);
+	sp->coordinates()->setGridLines(sum!=Qwt3D::NOSIDEGRID, true, sum);
 	sp->updateGL();
 	emit modified();
 }
@@ -1939,7 +1819,7 @@ void Graph3D::setGrid(int grids)
 	if (!sp)
 		return;
 
-	sp->coordinates()->setGridLines(true, false, grids);
+	sp->coordinates()->setGridLines(true, true, grids);
 }
 
 void Graph3D::setLeftGrid(bool b)
@@ -1976,12 +1856,19 @@ void Graph3D::print()
         printer.setOrientation(QPrinter::Portrait);
 	printer.setColorMode (QPrinter::Color);
 	printer.setFullPage(false);
-	if (printer.setup()){
-        QImage im = sp->grabFrameBuffer(true);
-        QPainter paint(&printer);
-        paint.drawImage(printer.pageRect(), im);
-        paint.end();
-	}
+	if (printer.setup())
+		print(&printer);
+}
+
+void Graph3D::print(QPrinter *printer)
+{
+	if (!printer)
+		return;
+
+	QRect rect = printer->pageRect();
+	QPainter paint(printer);
+	paint.drawPixmap(rect, sp->renderPixmap(rect.width(), rect.height()));
+	paint.end();
 }
 
 void Graph3D::copyImage()
@@ -2051,8 +1938,75 @@ void Graph3D::exportImage(const QString& fileName, int quality, bool transparent
 	int dpm = (int)ceil(100.0/2.54*dpi);
 	image.setDotsPerMeterX(dpm);
 	image.setDotsPerMeterY(dpm);
-	image.save(fileName, 0, quality);
+	if (fileName.endsWith(".odf")){
+		QTextDocument *document = new QTextDocument();
+		QTextCursor cursor = QTextCursor(document);
+		cursor.movePosition(QTextCursor::End);
+		cursor.insertText(objectName());
+		cursor.insertBlock();
+		cursor.insertImage(image);
 
+		QTextDocumentWriter writer(fileName);
+		writer.write(document);
+	} else
+		image.save(fileName, 0, quality);
+}
+
+
+void Graph3D::exportImage(QTextDocument *document, int, bool transparent,
+						int dpi, const QSizeF& customSize, int unit, double fontsFactor)
+{
+	if (!document)
+		return;
+
+    if (!dpi)
+		dpi = logicalDpiX();
+
+	QSize size = this->size();
+	if (customSize.isValid())
+		size = Graph::customPrintSize(customSize, unit, dpi);
+
+	if (fontsFactor == 0.0)
+		fontsFactor = (double)size.height()/(double)this->height();
+
+	scaleFonts(fontsFactor);
+
+	QPixmap pic = sp->renderPixmap(size.width(), size.height());
+
+	scaleFonts(1.0/fontsFactor);
+
+	sp->updateData();
+	QImage image = pic.toImage();
+
+	if (transparent){
+		QBitmap mask(size);
+		mask.fill(Qt::color1);
+		QPainter p(&mask);
+		p.setPen(Qt::color0);
+
+		QRgb backgroundPixel = QColor(Qt::white).rgb ();
+		for (int y = 0; y < image.height(); y++){
+			for (int x = 0; x < image.width(); x++){
+				QRgb rgb = image.pixel(x, y);
+				if (rgb == backgroundPixel) // we want the frame transparent
+					p.drawPoint(x, y);
+			}
+		}
+		p.end();
+		pic.setMask(mask);
+		image = pic.toImage();
+	}
+
+	int dpm = (int)ceil(100.0/2.54*dpi);
+	image.setDotsPerMeterX(dpm);
+	image.setDotsPerMeterY(dpm);
+
+	QTextCursor cursor = QTextCursor(document);
+	cursor.movePosition(QTextCursor::End);
+	cursor.insertBlock();
+	cursor.insertText(objectName());
+	cursor.insertBlock();
+	cursor.insertImage(image);
 }
 
 void Graph3D::exportPDF(const QString& fileName)
@@ -2060,7 +2014,8 @@ void Graph3D::exportPDF(const QString& fileName)
 	exportVector(fileName);
 }
 
-void Graph3D::exportVector(const QString& fileName, int textExportMode, int sortMode)
+void Graph3D::exportVector(const QString& fileName, int textExportMode, int sortMode,
+				const QSizeF& customSize, int unit, double fontsFactor)
 {
 	if ( fileName.isEmpty() ){
 		QMessageBox::critical(this, tr("QtiPlot - Error"), tr("Please provide a valid file name!"));
@@ -2077,14 +2032,27 @@ void Graph3D::exportVector(const QString& fileName, int textExportMode, int sort
     else if (fileName.endsWith(".pgf", Qt::CaseInsensitive))
         format = "PGF";
 
+	QSize cs = Graph::customPrintSize(customSize, unit, 72);
+	QSize size = this->size();
+	if (customSize.isValid())
+		size = cs;
+
+	if (fontsFactor == 0.0)
+		fontsFactor = (double)size.height()/(double)this->height();
+
     VectorWriter * gl2ps = (VectorWriter*)IO::outputHandler(format);
     if (gl2ps){
 		gl2ps->setTextMode((VectorWriter::TEXTMODE)textExportMode);
 		gl2ps->setLandscape(VectorWriter::OFF);
 		gl2ps->setSortMode((VectorWriter::SORTMODE)sortMode);
+		gl2ps->setExportSize(cs);
 	}
 
+	scaleFonts(fontsFactor);
+
 	IO::save(sp, fileName, format);
+
+	scaleFonts(1.0/fontsFactor);
 }
 
 void Graph3D::exportToFile(const QString& fileName)
@@ -2546,6 +2514,25 @@ void Graph3D::save(const QString &fn, const QString &geometry, bool)
 	t << "Orthogonal\t" + QString::number(sp->ortho())+"\n";
 	if (d_color_map_file.isEmpty())
 		t << ColorMapEditor::saveToXmlString(d_color_map);
+
+
+	t << "axisType\t" << scaleType[0] << "\t" << scaleType[1] << "\t" << scaleType[2] << "\n";
+
+	Qwt3D::GridLine gridLine = sp->coordinates()->majorGridLine(Qwt3D::X1);
+	RGBA color = gridLine.color_;
+	QColor c = GL2Qt(color.r, color.g, color.b);
+
+	t << "<Grid>\n";
+	t << "\t<Major>" + QString::number(gridLine.visible_) + "\t" + c.name() + "\t";
+	t << QString::number(gridLine.style_) + "\t" + QString::number(gridLine.width_) + "</Major>\n";
+
+	gridLine = sp->coordinates()->minorGridLine(Qwt3D::X1);
+	color = gridLine.color_;
+	c = GL2Qt(color.r, color.g, color.b);
+	t << "\t<Minor>" + QString::number(gridLine.visible_) + "\t" + c.name() + "\t";
+	t << QString::number(gridLine.style_) + "\t" + QString::number(gridLine.width_) + "</Minor>\n";
+	t << "</Grid>\n";
+
 	t << "</SurfacePlot>\n";
 }
 
@@ -2789,18 +2776,21 @@ void Graph3D::findBestLayout()
 {
   	double start, end;
   	sp->coordinates()->axes[X1].limits (start, end);
-  	double xScale = 1/fabs(end-start);
+  	double xScale = 1/fabs(end - start);
 
   	sp->coordinates()->axes[Y1].limits (start, end);
-  	double yScale = 1/fabs(end-start);
+  	double yScale = 1/fabs(end - start);
 
   	sp->coordinates()->axes[Z1].limits (start, end);
-  	double zScale = 1/fabs(end-start);
+  	double zScale = 1/fabs(end - start);
 
-  	double d = (sp->hull().maxVertex-sp->hull().minVertex).length();
+  	double d = (sp->hull().maxVertex - sp->hull().minVertex).length();
   	sp->setScale(xScale, yScale, zScale);
+
   	sp->setZoom(d/sqrt(3.));
 
+	sp->setShift(0, 0, 0);
+
   	double majl = 0.1/yScale;
   	setAxisTickLength(0, majl, 0.6*majl);
   	majl = 0.1/xScale;
@@ -2891,7 +2881,6 @@ void Graph3D::copy(Graph3D* g)
 	setNumbersColor(g->numColor());
 	setLabelsColor(g->labelColor());
 	setBackgroundColor(g->bgColor());
-	setGridColor(g->gridColor());
 
 	setAxesLabels(g->axesLabels());
 	setTicks(g->scaleTicks());
@@ -2907,9 +2896,25 @@ void Graph3D::copy(Graph3D* g)
 	setShift(g->xShift(),g->yShift(),g->zShift());
 	setMeshLineWidth(g->meshLineWidth());
 
+	for (int i = 0; i < 3; i++)
+		scaleType[i] = g->axisType(i);
+
+	resetAxesType();
+
+	CoordinateSystem *coord = sp->coordinates();
+	CoordinateSystem *gcoord = g->surface()->coordinates();
+	for (int i = 0; i < 12; i++){
+		coord->axes[i].setMajors(gcoord->axes[i].majors());
+		coord->axes[i].setMinors(gcoord->axes[i].minors());
+
+		Qwt3D::AXIS axis = (Qwt3D::AXIS)i;
+		coord->setMajorGridLines(axis, gcoord->majorGridLine(axis));
+		coord->setMinorGridLines(axis, gcoord->minorGridLine(axis));
+	}
+
 	bool smooth = g->antialiasing();
     sp->setSmoothMesh(smooth);
-	sp->coordinates()->setLineSmooth(smooth);
+	coord->setLineSmooth(smooth);
 
 	setOrthogonal(g->isOrthogonal());
 
@@ -2918,6 +2923,49 @@ void Graph3D::copy(Graph3D* g)
 	animate(g->isAnimated());
 }
 
+void Graph3D::setAxisType(int axis, int type)
+{
+	if (axis >= 0 && axis < 3)
+		scaleType[axis] = type;
+}
+
+void Graph3D::resetAxesType()
+{
+	CoordinateSystem *coord = sp->coordinates();
+
+	int *majorTics = new int[12];
+	int *minorTics = new int[12];
+	for (int i = 0; i < 12; i++){
+		majorTics[i] = coord->axes[i].majors();
+		minorTics[i] = coord->axes[i].minors();
+	}
+
+	SCALETYPE type = (SCALETYPE)scaleType[0];
+	coord->axes[X1].setScale(type);
+	coord->axes[X2].setScale(type);
+	coord->axes[X3].setScale(type);
+	coord->axes[X4].setScale(type);
+	type = (SCALETYPE)scaleType[1];
+	coord->axes[Y1].setScale(type);
+	coord->axes[Y2].setScale(type);
+	coord->axes[Y3].setScale(type);
+	coord->axes[Y4].setScale(type);
+	type = (SCALETYPE)scaleType[2];
+	coord->axes[Z1].setScale(type);
+	coord->axes[Z2].setScale(type);
+	coord->axes[Z3].setScale(type);
+	coord->axes[Z4].setScale(type);
+
+	for (int i = 0; i < 12; i++){
+		coord->axes[i].setMajors(majorTics[i]);
+		coord->axes[i].setMinors(minorTics[i]);
+	}
+
+	delete [] majorTics;
+	delete [] minorTics;
+
+}
+
 Graph3D* Graph3D::restore(ApplicationWindow* app, const QStringList &lst, int fileVersion)
 {
 	QStringList fList=lst[0].split("\t");
@@ -3004,7 +3052,7 @@ Graph3D* Graph3D::restore(ApplicationWindow* app, const QStringList &lst, int fi
 			plot->setDataColors(QColor(colors[7]), QColor(colors[8]));
 	}
 
-	fList=lst[7].split("\t", QString::SkipEmptyParts);
+	fList = lst[7].split("\t", QString::SkipEmptyParts);
 	fList.pop_front();
 	plot->setAxesLabels(fList);
 
@@ -3071,10 +3119,76 @@ Graph3D* Graph3D::restore(ApplicationWindow* app, const QStringList &lst, int fi
 		colorMap.addColorStop(l[0].toDouble(), QColor(l[1]));
 	}
 	plot->setDataColorMap(colorMap);
+
+	line.next();
+	s = line.next();
+
+	fList = s.split("\t", QString::SkipEmptyParts);
+	if (fList.size() == 4 && fList[0] == "axisType"){
+		fList.removeFirst();
+		for (int i = 0; i < 3; i++)
+			plot->setAxisType(i, fList[i].toInt());
+
+		plot->resetAxesType();
+	}
+
+	if (line.hasNext()){
+		s = line.next();
+		if (s == "<Grid>" && line.hasNext()){
+			s = line.next().stripWhiteSpace();
+			if (s.contains("<Major>")){
+				fList = s.remove("<Major>").remove("</Major>").split("\t", QString::SkipEmptyParts);
+				if (fList.size() == 4){
+					Qwt3D::GridLine line(fList[0].toInt(), Qt2GL(QColor(fList[1])), (Qwt3D::LINESTYLE)fList[2].toInt(), fList[3].toDouble());
+					for (int i = 0; i < 12; i++)
+						plot->coordinateSystem()->setMajorGridLines((Qwt3D::AXIS)i, line);
+				}
+			}
+
+			if (line.hasNext())
+				s = line.next().stripWhiteSpace();
+			if (s.contains("<Minor>")){
+				fList = s.remove("<Minor>").remove("</Minor>").split("\t", QString::SkipEmptyParts);
+				if (fList.size() == 4){
+					Qwt3D::GridLine line(fList[0].toInt(), Qt2GL(QColor(fList[1])), (Qwt3D::LINESTYLE)fList[2].toInt(), fList[3].toDouble());
+					for (int i = 0; i < 12; i++)
+						plot->coordinateSystem()->setMinorGridLines((Qwt3D::AXIS)i, line);
+				}
+			}
+		}
+	}
+
 	plot->update();
 	return plot;
 }
 
+void Graph3D::dragEnterEvent( QDragEnterEvent* e )
+{
+	if (e->mimeData()->hasFormat("text/plain"))
+		e->acceptProposedAction();
+}
+
+void Graph3D::dropEvent(QDropEvent* event)
+{
+	Table *t = qobject_cast<Table*>(event->source());
+	if (t){
+		QStringList columns = event->mimeData()->text().split("\n");
+		if (columns.isEmpty())
+			return;
+
+		int zcol = t->colIndex(columns[0]);
+		if (t->colPlotDesignation(zcol) != Table::Z)
+			return;
+
+		addData(t, t->colX(zcol), t->colY(zcol), zcol, Trajectory);
+		return;
+	}
+
+	Matrix *m = qobject_cast<Matrix*>(event->source());
+	if (m)
+		addMatrixData(m);
+}
+
 Graph3D::~Graph3D()
 {
 	if (d_surface)
