@@ -31,6 +31,7 @@
 #include "PythonSyntaxHighlighter.h"
 #include "FindReplaceDialog.h"
 #include <ApplicationWindow.h>
+#include <pixmaps.h>
 
 #include <QAction>
 #include <QMenu>
@@ -49,11 +50,13 @@
 
 ScriptEdit::ScriptEdit(ScriptingEnv *env, QWidget *parent, const char *name)
   : QTextEdit(parent, name), scripted(env), d_error(false), d_completer(0),
-  d_file_name(QString::null), d_search_string(QString::null)
+  d_file_name(QString::null), d_search_string(QString::null), d_output_widget(NULL)
 {
 	myScript = scriptEnv->newScript("", this, name);
 	connect(myScript, SIGNAL(error(const QString&, const QString&, int)), this, SLOT(insertErrorMsg(const QString&)));
 	connect(myScript, SIGNAL(print(const QString&)), this, SLOT(scriptPrint(const QString&)));
+	connect(myScript, SIGNAL(error(const QString&, const QString&, int)),
+			this, SIGNAL(error(const QString&, const QString&, int)));
 
 	setLineWrapMode(NoWrap);
 	setUndoRedoEnabled(true);
@@ -76,7 +79,7 @@ ScriptEdit::ScriptEdit(ScriptingEnv *env, QWidget *parent, const char *name)
 	actionExecute->setShortcut( tr("Ctrl+J") );
 	connect(actionExecute, SIGNAL(activated()), this, SLOT(execute()));
 
-	actionExecuteAll = new QAction(tr("Execute &All"), this);
+	actionExecuteAll = new QAction(QIcon(QPixmap(play_xpm)), tr("Execute &All"), this);
 	actionExecuteAll->setShortcut( tr("Ctrl+Shift+J") );
 	connect(actionExecuteAll, SIGNAL(activated()), this, SLOT(executeAll()));
 
@@ -84,54 +87,42 @@ ScriptEdit::ScriptEdit(ScriptingEnv *env, QWidget *parent, const char *name)
 	actionEval->setShortcut( tr("Ctrl+Return") );
 	connect(actionEval, SIGNAL(activated()), this, SLOT(evaluate()));
 
-	actionPrint = new QAction(tr("&Print"), this);
+	actionPrint = new QAction(QPixmap(fileprint_xpm), tr("&Print"), this);
 	connect(actionPrint, SIGNAL(activated()), this, SLOT(print()));
 
-	actionImport = new QAction(tr("&Import..."), this);
+	actionImport = new QAction(QPixmap(fileopen_xpm), tr("&Import..."), this);
 	actionImport->setShortcut(QKeySequence(Qt::CTRL+Qt::ALT+Qt::Key_O));
 	connect(actionImport, SIGNAL(activated()), this, SLOT(importASCII()));
 
 	QShortcut *accelImport = new QShortcut(actionImport->shortcut(), this);
 	connect(accelImport, SIGNAL(activated()), this, SLOT(importASCII()));
 
-	actionSave = new QAction(tr("&Save"), this);
+	actionSave = new QAction(QPixmap(filesave_xpm), tr("&Save"), this);
 	actionSave->setShortcut(QKeySequence(Qt::CTRL+Qt::ALT+Qt::Key_S));
 	connect(actionSave, SIGNAL(activated()), this, SLOT(save()));
 
 	QShortcut *accelSave = new QShortcut(actionSave->shortcut(), this);
 	connect(accelSave, SIGNAL(activated()), this, SLOT(save()));
 
-	actionExport = new QAction(tr("Sa&ve as..."), this);
+	actionExport = new QAction(QIcon(QPixmap(filesaveas_xpm)), tr("Sa&ve as..."), this);
 	connect(actionExport, SIGNAL(activated()), this, SLOT(exportASCII()));
 
-	actionFind = new QAction(tr("&Find..."), this);
+	actionFind = new QAction(QPixmap(find_xpm), tr("&Find..."), this);
 	actionFind->setShortcut(QKeySequence(Qt::CTRL+Qt::ALT+Qt::Key_F));
 	connect(actionFind, SIGNAL(activated()), this, SLOT(showFindDialog()));
 
-	QShortcut *accelFind = new QShortcut(actionFind->shortcut(), this);
-	connect(accelFind, SIGNAL(activated()), this, SLOT(showFindDialog()));
-
-	actionReplace = new QAction(tr("&Replace..."), this);
+	actionReplace = new QAction(QPixmap(replace_xpm), tr("&Replace..."), this);
 	actionReplace->setShortcut(QKeySequence(Qt::CTRL+Qt::ALT+Qt::Key_R));
 	connect(actionReplace, SIGNAL(activated()), this, SLOT(replace()));
 
-	QShortcut *accelReplace = new QShortcut(actionReplace->shortcut(), this);
-	connect(accelReplace, SIGNAL(activated()), this, SLOT(replace()));
-
-	actionFindNext = new QAction(tr("&Find next"), this);
+	actionFindNext = new QAction(QPixmap(find_next_xpm), tr("&Find next"), this);
 	actionFindNext->setShortcut(QKeySequence(Qt::Key_F3));
 	connect(actionFindNext, SIGNAL(activated()), this, SLOT(findNext()));
 
-	QShortcut *accelFindNext = new QShortcut(actionFindNext->shortcut(), this);
-	connect(accelFindNext, SIGNAL(activated()), this, SLOT(findNext()));
-
-	actionFindPrevious = new QAction(tr("&Find previous"), this);
+	actionFindPrevious = new QAction(QPixmap(find_previous_xpm), tr("&Find previous"), this);
 	actionFindPrevious->setShortcut(QKeySequence(Qt::Key_F4));
 	connect(actionFindPrevious, SIGNAL(activated()), this, SLOT(findPrevious()));
 
-	QShortcut *accelFindPrevious = new QShortcut(actionFindPrevious->shortcut(), this);
-	connect(accelFindPrevious, SIGNAL(activated()), this, SLOT(findPrevious()));
-
 	functionsMenu = new QMenu(this);
 	Q_CHECK_PTR(functionsMenu);
 	connect(functionsMenu, SIGNAL(triggered(QAction *)), this, SLOT(insertFunction(QAction *)));
@@ -197,12 +188,12 @@ void ScriptEdit::keyPressEvent(QKeyEvent *e)
      QString completionPrefix = textUnderCursor();
 
      if (!isShortcut && (hasModifier || e->text().isEmpty()|| completionPrefix.length() < 2
-                       || eow.contains(e->text().right(1)))) {
+		 || eow.contains(e->text().right(1)))){
          d_completer->popup()->hide();
          return;
      }
 
-     if (completionPrefix != d_completer->completionPrefix()) {
+     if (completionPrefix != d_completer->completionPrefix()){
          d_completer->setCompletionPrefix(completionPrefix);
          d_completer->popup()->setCurrentIndex(d_completer->completionModel()->index(0, 0));
      }
@@ -233,18 +224,39 @@ void ScriptEdit::contextMenuEvent(QContextMenuEvent *e)
 	menu->addAction(actionSave);
 	menu->addAction(actionExport);
 	menu->insertSeparator();
+
+	Note *sp = qobject_cast<Note*>(myScript->context());
+	if (sp){
+		QAction *actionRenameTab = new QAction(tr("Rena&me Tab..."), menu);
+		connect(actionRenameTab, SIGNAL(activated()), sp, SLOT(renameCurrentTab()));
+		menu->addAction(actionRenameTab);
+
+		QAction *actionAddTab = new QAction(QIcon(QPixmap(plus_xpm)), tr("A&dd Tab"), menu);
+		connect(actionAddTab, SIGNAL(activated()), sp, SLOT(addTab()));
+		menu->addAction(actionAddTab);
+
+		if (sp->tabs() > 1){
+			QAction *actionRemoveTab = new QAction(QIcon(QPixmap(delete_xpm)), tr("C&lose Tab"), menu);
+			connect(actionRemoveTab, SIGNAL(activated()), sp, SLOT(removeTab()));
+			menu->addAction(actionRemoveTab);
+		}
+
+		menu->insertSeparator();
+	}
+
 	if (!emptyText){
 		menu->addAction(actionExecute);
 		menu->addAction(actionExecuteAll);
 		menu->addAction(actionEval);
 	}
-	if (parent()->isA("Note")){
-		Note *sp = (Note*) parent();
+
+	if (sp){
 		QAction *actionAutoexec = new QAction(tr("Auto&exec"), menu);
 		actionAutoexec->setToggleAction(true);
 		actionAutoexec->setOn(sp->autoexec());
 		connect(actionAutoexec, SIGNAL(toggled(bool)), sp, SLOT(setAutoexec(bool)));
 		menu->addAction(actionAutoexec);
+		menu->insertSeparator();
 	}
 
 	functionsMenu->clear();
@@ -291,7 +303,7 @@ void ScriptEdit::insertErrorMsg(const QString &message)
 	console->setPlainText(err);
 	if (!console->isVisible())
 		((QDockWidget *)console->parent())->show();
-#elif
+#else
 	int start = printCursor.position();
 	printCursor.insertText(err);
 	printCursor.setPosition(start, QTextCursor::KeepAnchor);
@@ -392,11 +404,9 @@ void ScriptEdit::evaluate()
 
 	myScript->setName(fname);
 	myScript->setCode(codeCursor.selectedText().replace(QChar::ParagraphSeparator,"\n"));
-	//printCursor.setPosition(codeCursor.selectionEnd(), QTextCursor::MoveAnchor);
-	//printCursor.movePosition(QTextCursor::EndOfLine, QTextCursor::MoveAnchor);
 	QVariant res = myScript->eval();
 
-	if (res.isValid())
+	if (res.isValid() && !myScript->code().isEmpty())
 		if (!res.isNull() && res.canConvert(QVariant::String)){
 			QString strVal = res.toString();
 			strVal.replace("\n", "\n#> ");
@@ -415,7 +425,6 @@ void ScriptEdit::exportPDF(const QString& fileName)
 {
 	QTextDocument *doc = document();
 	QPrinter printer;
-	printer.setColorMode(QPrinter::GrayScale);
 	printer.setCreator("QtiPlot");
     printer.setOutputFormat(QPrinter::PdfFormat);
     printer.setOutputFileName(fileName);
@@ -426,22 +435,29 @@ void ScriptEdit::print()
 {
 	QTextDocument *doc = document();
 	QPrinter printer;
-	printer.setColorMode(QPrinter::GrayScale);
 	QPrintDialog printDialog(&printer);
 	// TODO: Write a dialog to use more features of Qt4's QPrinter class
 	if (printDialog.exec() == QDialog::Accepted)
 		doc->print(&printer);
 }
 
+void ScriptEdit::print(QPrinter *printer)
+{
+	if (!printer)
+		return;
+
+	document()->print(printer);
+}
+
 QString ScriptEdit::importASCII(const QString &filename)
 {
-	QString filter = tr("Text") + " (*.txt *.TXT);;";
-	filter += scriptEnv->fileFilter();
+	QString filter = scriptEnv->fileFilter();
+	filter += tr("Text") + " (*.txt *.TXT);;";
 	filter += tr("All Files")+" (*)";
 
 	QString f;
 	if (filename.isEmpty())
-		f = QFileDialog::getOpenFileName(this, tr("QtiPlot - Import Text From File"), scriptsDirPath, filter);
+		f = ApplicationWindow::getFileName(this, tr("QtiPlot - Import Text From File"), scriptsDirPath, filter, 0, false);
 	else
 		f = filename;
 	if (f.isEmpty()) return QString::null;
@@ -491,16 +507,14 @@ QString ScriptEdit::exportASCII(const QString &filename)
 	QString selectedFilter;
 	QString fn;
 	if (filename.isEmpty())
-		fn = QFileDialog::getSaveFileName(this, tr("Save Text to File"), scriptsDirPath, filter, &selectedFilter);
+		fn = ApplicationWindow::getFileName(this, tr("Save Text to File"), scriptsDirPath, filter, &selectedFilter);
 	else
 		fn = filename;
 
 	if (!fn.isEmpty()){
 		QFileInfo fi(fn);
 		scriptsDirPath = fi.absolutePath();
-
-		QString baseName = fi.fileName();
-		if (!baseName.contains(".")){
+		if (!fn.endsWith(selectedFilter.remove("*"), Qt::CaseInsensitive)){
 			if (selectedFilter.contains(".txt"))
 				fn.append(".txt");
 			else if (selectedFilter.contains(".py"))
@@ -532,7 +546,7 @@ void ScriptEdit::setFileName(const QString& fn)
 	d_file_name = fn;
 	Note *note = qobject_cast<Note *>(myScript->context());
 	if (note)
-		note->setWindowLabel(d_file_name);
+		note->renameTab(note->indexOf(this), QFileInfo(d_file_name).fileName());
 }
 
 void ScriptEdit::updateIndentation()
@@ -540,6 +554,7 @@ void ScriptEdit::updateIndentation()
 	QTextCursor cursor = textCursor();
 	QTextBlock para = cursor.block();
 	QString prev = para.previous().text();
+
 	int i;
 	for (i=0; prev[i].isSpace(); i++);
 	QString indent = prev.mid(0, i);
@@ -667,6 +682,9 @@ void ScriptEdit::clearErrorHighlighting()
 	codeCursor.movePosition(QTextCursor::Start, QTextCursor::MoveAnchor);
 	codeCursor.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
 	codeCursor.mergeBlockFormat(d_fmt_default);
+
+	if (d_output_widget)
+		d_output_widget->clear();
 }
 
 void ScriptEdit::highlightErrorLine(int offset)
@@ -696,6 +714,15 @@ void ScriptEdit::highlightErrorLine(int offset)
 	setTextCursor(codeCursor);
 }
 
+void ScriptEdit::redirectOutputTo(QTextEdit *te)
+{
+	d_output_widget = te;
+	if (d_output_widget)
+		printCursor = QTextCursor(d_output_widget->textCursor());
+	else
+		printCursor = textCursor();
+}
+
 ScriptEdit::~ScriptEdit()
 {
  #ifdef SCRIPTING_PYTHON
